#!/usr/bin/python
## ################################################################# ##
## (C) SCRT - Information Security, 2007 - 2008 // author: ~SaD~     ##
## ################################################################# ##
## This program is free software: you can redistribute it and/or     ##
## modify it under the terms of the GNU General Public License as    ##
## published by the Free Software Foundation, either version 3 of    ##
## the License, or (at your option) any later version.               ##
## This program is distributed in the hope that it will be useful,   ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of    ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     ##
## GNU General Public License for more details.                      ##
## You should have received a copy of the GNU General Public License ##
## along with this program. If not, see http://www.gnu.org/licenses. ##
## ################################################################# ##
## last mod: 2008-02-18

import sys
import os.path
from optparse import OptionParser
from webshag.cli import cli
from webshag.core import core_utilities

## ################################################################# ##
## MODULE CONSTANTS
## ################################################################# ##

WEBSHAG = u'webshag'
WEBSHAG_VERSION = u'1.00'

MODULE_PSCAN = 'pscan'
MODULE_INFO = 'info'
MODULE_SPIDER = 'spider'
MODULE_USCAN = 'uscan'
MODULE_FUZZ = 'fuzz'
MODULES = [MODULE_PSCAN, MODULE_INFO, MODULE_SPIDER, MODULE_USCAN, MODULE_FUZZ]
FUZZ_MODE_GEN = u'gen'
FUZZ_MODE_LIST = u'list'
FUZZ_MODES = [FUZZ_MODE_GEN, FUZZ_MODE_LIST] 
OUTPUT_XML = u'xml'
OUTPUT_HTML = u'html'
OUTPUT_TXT = u'txt'
DEFAULT_MODULE = MODULE_USCAN
DEFAULT_SKIP = u''
DEFAULT_SERVER = u''
DEFAULT_PORT = u'80'
DEFAULT_ROOT = u'/'
DEFAULT_FUZZ_MODE = FUZZ_MODE_LIST
DEFAULT_FUZZ_EXTENSION = u'.html'
DEFAULT_FUZZ_GENERATOR = u''
DEFAULT_SPIDER_INIT = u'/'
DEFAULT_OUTPUT = OUTPUT_HTML
DEFAULT_OUTPUT_FILE = u'webshag_report.html'
OUTPUT = [OUTPUT_XML, OUTPUT_HTML, OUTPUT_TXT]

## ################################################################# ##
## MAIN
## ################################################################# ##

def main():
    
    usage_string = "usage: %prog [-U | -C [options] target(s)]"
    version_string = '%prog ' + WEBSHAG_VERSION
    parser = OptionParser(usage=usage_string, version=version_string)
    
    parser.add_option('-C', dest='cli', action='store_true', default=False, help='Use the Command Line Interface (CLI) mode')
    parser.add_option('-U', dest='update', action='store_true', default=False, help='Update the URL scanner databases and exit')
    
    parser.add_option('-m', dest='module', default=DEFAULT_MODULE, help='*CLI only* Use MODULE [' + MODULE_PSCAN + '|' + MODULE_INFO + '|' + MODULE_SPIDER + '|' + MODULE_USCAN + '|' + MODULE_FUZZ + ']. (default: ' + DEFAULT_MODULE + ')')
    parser.add_option('-p', dest='port', default=DEFAULT_PORT, help='*CLI only* Set target port to PORT. For modules ' + MODULE_USCAN + ' and ' + MODULE_FUZZ + ' PORT can be a list of ports [port1,port2,...]. (default: ' + DEFAULT_PORT + ')')
    parser.add_option('-r', dest='root', default=DEFAULT_ROOT, help='*CLI only* Set root directory to ROOT. For modules ' + MODULE_USCAN + ' and ' + MODULE_FUZZ + ' ROOT can be a list of directories [/root1/,/root2/,...]. (default: ' + DEFAULT_ROOT + ')')
    
    parser.add_option('-k', dest='skip', default=DEFAULT_SKIP, help='*CLI (' + MODULE_USCAN + ') only* Set a false positive detection string')
    parser.add_option('-s', dest='server', default=DEFAULT_SERVER, help='*CLI (' + MODULE_USCAN + ') only* Bypass server detection and force server as SERVER')
    
    parser.add_option('-i', dest='spider_init', default=DEFAULT_SPIDER_INIT, help='*CLI (' + MODULE_SPIDER + ') only* Set spider initial crawling page (default: ' + DEFAULT_SPIDER_INIT + ')')
    
    parser.add_option('-n', dest='fuzz_mode', default=DEFAULT_FUZZ_MODE, help='*CLI (' + MODULE_FUZZ + ') only* Choose the fuzzing mode [' + FUZZ_MODE_LIST + '|' + FUZZ_MODE_GEN + ']. (default: ' + DEFAULT_FUZZ_MODE + ')')
    parser.add_option('-e', dest='fuzz_ext', default=DEFAULT_FUZZ_EXTENSION, help='*CLI (' + MODULE_FUZZ + ' / ' + FUZZ_MODE_LIST + ') only* Set a default extension to append to all filenames. (default: ' + DEFAULT_FUZZ_EXTENSION + ')')
    parser.add_option('-g', dest='fuzz_gen', default=DEFAULT_FUZZ_GENERATOR, help='*CLI (' + MODULE_FUZZ + ' / ' + FUZZ_MODE_GEN + ') only* Set the filename generator expression. Refer to documentation for syntax reference. (default: ' + DEFAULT_FUZZ_GENERATOR + ')')

    parser.add_option('-x', dest='export', action='store_true', default=False, help='*CLI only* Export a report summarizing results.')
    parser.add_option('-o', dest='output', default=DEFAULT_OUTPUT, help='*CLI only* Set the format of the exported report. [' + OUTPUT_XML + '|' + OUTPUT_HTML + '|' + OUTPUT_TXT + ']. (default: ' + DEFAULT_OUTPUT + ')')
    parser.add_option('-f', dest='output_file', default=DEFAULT_OUTPUT_FILE, help='*CLI only* Write report to FILE. (default: ' + DEFAULT_OUTPUT_FILE + ')')
    
    (options, arguments) = parser.parse_args()
    
    
    if options.update and options.cli:
        
        parser.error(u'-U is a standalone option, it cannot be combined with -C and ignores any other option or argument')
        
    elif options.update and not options.cli:
        
        cli.update_databases()
    
    elif not options.cli:
        
        try:
            from webshag.gui import gui
            gui.run()
        except ImportError:
            sys.exit('Impossible to locate all required modules!\nCheck that you have wxPython installed or use ' + WEBSHAG + ' in CLI mode.')
            
        
    elif options.cli and options.module != None and options.module in MODULES and len(arguments) > 0:
        
        
        target = ','.join(a.strip() for a in arguments)
        
        if core_utilities.check_host_list(target):
            check_target = True
            nb_target = len(target.split(u','))
            
        else:
            check_target = False
        
        
        if check_target:
            
            # ###################################################### #
            #                APPLICATION SETTINGS                    #
            # ###################################################### #
            
            # ###################################################### #
            # MODULE = PSCAN                                         #
            # ###################################################### #
            if options.module == MODULE_PSCAN:
                
                if nb_target == 1:
                    
                    module_settings = {}
                    module_settings[u'MODULE'] = MODULE_PSCAN
                    module_settings[u'HOST'] = target
                    
                    setup = True
                    
                else: 
                    setup = False
                    error_message = u'Too many targets specified for module ' + options.module
            
            # ###################################################### #
            # MODULE = INFO                                          #
            # ###################################################### #
            elif options.module == MODULE_INFO:
                
                if nb_target == 1:
                    
                    module_settings = {}
                    module_settings[u'MODULE'] = MODULE_INFO
                    module_settings[u'HOST'] = target
                    
                    setup = True
                    
                else:
                    setup = False
                    error_message = u'Too many targets specified for module ' + options.module
            
            # ###################################################### #
            # MODULE = SPIDER                                        #
            # ###################################################### #
            elif options.module == MODULE_SPIDER:
                
                if nb_target == 1:
                    
                    if core_utilities.check_spider_root(options.spider_init) and core_utilities.check_port_string(options.port):
                    
                        module_settings = {}
                        module_settings[u'MODULE'] = MODULE_SPIDER
                        module_settings[u'HOST'] = target
                        module_settings[u'PORT'] = options.port
                        module_settings[u'ROOT'] = options.spider_init
                        
                        setup = True
                        
                    else:
                        setup = False
                        error_message = u'Invalid parameter values for module ' + options.module
                else:
                    setup = False
                    error_message = u'Too many targets specified for module ' + options.module
                
            
            # ###################################################### #
            # MODULE = USCAN                                         #
            # ###################################################### #
            elif options.module == MODULE_USCAN:
                
                if core_utilities.check_port_list(options.port) and core_utilities.check_root_list(options.root) and core_utilities.check_ascii_string(options.server) and core_utilities.check_generic_string(options.skip):
                    
                    module_settings = {}
                    module_settings[u'MODULE'] = MODULE_USCAN
                    module_settings[u'HOSTS'] = target
                    module_settings[u'PORTS'] = options.port
                    module_settings[u'ROOTS'] = options.root
                    module_settings[u'SKIP'] = options.skip
                    module_settings[u'SERVER'] = options.server
                    
                    setup = True
                
                else:
                    setup = False
                    error_message = u'Invalid parameter values for module ' + options.module
                
                
            # ###################################################### #
            # MODULE = FUZZ                                          #
            # ###################################################### #
            elif options.module == MODULE_FUZZ:
                
                if core_utilities.check_port_list(options.port) and core_utilities.check_root_list(options.root) and options.fuzz_mode in FUZZ_MODES:
                    
                    if options.fuzz_mode == FUZZ_MODE_LIST and core_utilities.check_extension(options.fuzz_ext):
                        
                        module_settings = {}
                        module_settings[u'MODULE'] = MODULE_FUZZ
                        module_settings[u'HOSTS'] = target
                        module_settings[u'PORTS'] = options.port
                        module_settings[u'ROOTS'] = options.root
                        module_settings[u'MODE'] = (options.fuzz_mode, options.fuzz_ext)
                        
                        setup = True
                        
                    elif options.fuzz_mode == FUZZ_MODE_GEN and core_utilities.check_generic_string(options.fuzz_gen):
                        
                        module_settings = {}
                        module_settings[u'MODULE'] = MODULE_FUZZ
                        module_settings[u'HOSTS'] = target
                        module_settings[u'PORTS'] = options.port
                        module_settings[u'ROOTS'] = options.root
                        module_settings[u'MODE'] = (options.fuzz_mode, options.fuzz_gen)
                        
                        setup = True
                    
                    else:
                        setup = False
                        error_message = u'Invalid parameter values for module ' + options.module
                
                else:
                    setup = False
                    error_message = u'Invalid parameter values for module ' + options.module
            
            
                
            
            # ###################################################### #
            #                   EXPORT SETTINGS                      #
            # ###################################################### #
            
            if options.export:
                
                if options.output in OUTPUT and core_utilities.check_save_file_path(os.path.realpath(options.output_file)):
                    
                    export_settings = {}
                    export_settings[u'EXPORT'] = True
                    export_settings[u'FORMAT'] = options.output
                    export_settings[u'FILE'] = os.path.realpath(options.output_file)
                    
                    exp_setup = True
                
                else:
                    
                    exp_setup = False
                    exp_error_message = u'Invalid export parameter values'
                
            else:
                
                export_settings = {}
                export_settings[u'EXPORT'] = False
                export_settings[u'FORMAT'] = u''
                export_settings[u'FILE'] = u''
                
                exp_setup = True
                
            # ###################################################### #
            #                         RUNNING                        #
            # ###################################################### #
            
            if setup:
                if exp_setup:
                    cli.run(module_settings, export_settings)
                else:
                    parser.error(exp_error_message)
            else:
                parser.error(error_message)
        else: # target does not pass check
            parser.error("Invalid target(s) specified")
            
    else: # bad module or target
        parser.error("bad module or target missing")
    

if __name__ == "__main__":
    main()
