## ################################################################# ##
## (C) SCRT - Information Security, 2007 - 2008 // author: ~SaD~     ##
## ################################################################# ##
## This program is free software: you can redistribute it and/or     ##
## modify it under the terms of the GNU General Public License as    ##
## published by the Free Software Foundation, either version 3 of    ##
## the License, or (at your option) any later version.               ##
## This program is distributed in the hope that it will be useful,   ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of    ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     ##
## GNU General Public License for more details.                      ##
## You should have received a copy of the GNU General Public License ##
## along with this program. If not, see http://www.gnu.org/licenses. ##
## ################################################################# ##
## last mod: 2008-02-18

from __future__ import with_statement
from time import sleep
from threading import Thread, Lock
from webshag.core import core_utilities, core_file
from webshag.modules import module_info, module_uscan, module_pscan, module_fuzz, module_spider
from webshag.export import export
from webshag.update import update

## ################################################################# ##
## MODULE CONSTANTS
## ################################################################# ##

MODULE_PSCAN = 'pscan'
MODULE_INFO = 'info'
MODULE_SPIDER = 'spider'
MODULE_USCAN = 'uscan'
MODULE_FUZZ = 'fuzz'
MODULE_FUZZ_GEN = 'gen'
MODULE_FUZZ_LIST = 'list'
WEBSHAG = u'webshag'
WEBSHAG_VERSION = u'1.00'

## ################################################################# ##
## PUBLIC FUNCTIONS
## ################################################################# ##

def update_databases():
    
    success_nikto = update.update_nikto_database()
    __cli_print_update(u'Nikto', success_nikto)
    success_custom = update.update_custom_database()  
    __cli_print_update(u'SCRT', success_custom)

## ################################################################# ##

def run(module_settings, export_settings):
    
    results = []
    results_lock = Lock()
    switch = [True]
    switch_lock = Lock()
    
    __cli_print_start(module_settings, export_settings)
    
    
    runner = ModuleRunner(module_settings, results, results_lock, switch, switch_lock)
    runner.start()
    
    while runner.isAlive():
        try:
            # active wait #
            sleep(1)
            
        except KeyboardInterrupt:
            with switch_lock:
                switch[0] = False
    
    if switch[0] == True and export_settings[u'EXPORT'] == True:
        
        module = module_settings[u'MODULE']
        if module == MODULE_PSCAN:
            
            success = export.exp_report(export_settings[u'FILE'], export_settings[u'FORMAT'], pscanres=results)
            __cli_print_export(success, export_settings[u'FILE'])
            
        elif module == MODULE_INFO:
            
            success = export.exp_report(export_settings[u'FILE'], export_settings[u'FORMAT'], infores=results)
            __cli_print_export(success, export_settings[u'FILE'])
            
        elif module == MODULE_SPIDER:
            
            success = export.exp_report(export_settings[u'FILE'], export_settings[u'FORMAT'], spidres=results)
            __cli_print_export(success, export_settings[u'FILE'])
            
        elif module == MODULE_USCAN:
            
            success = export.exp_report(export_settings[u'FILE'], export_settings[u'FORMAT'], uscanres=results)
            __cli_print_export(success, export_settings[u'FILE'])
        
        elif module == MODULE_FUZZ:
            
            success = export.exp_report(export_settings[u'FILE'], export_settings[u'FORMAT'], fuzzres=results)
            __cli_print_export(success, export_settings[u'FILE'])
    else:
        pass
    
    
    __cli_print_end()


## ################################################################# ##
## PRIVATE FUNCTIONS
## ################################################################# ##

def __cli_print_start(module_settings, export_settings):
    
    module = module_settings[u'MODULE']

    print '~~~~~~~~~~~~~~~~~~~~~~~~~~ ## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
    print '% ' + WEBSHAG + ' ' + WEBSHAG_VERSION
    print '% Module: ' + module
    
    if module == MODULE_USCAN:
        print '% Host(s): ' + module_settings[u'HOSTS']
        print '% Port(s): ' + module_settings[u'PORTS']
        print '% Root(s): ' + module_settings[u'ROOTS']
    elif module == MODULE_FUZZ:
        print '% Host(s): ' + module_settings[u'HOSTS']
        print '% Port(s): ' + module_settings[u'PORTS']
        print '% Root(s): ' + module_settings[u'ROOTS']
        print '% Mode Settings: ' + module_settings[u'MODE'][0] + ' / ' + module_settings[u'MODE'][1]
    elif module == MODULE_PSCAN or module == MODULE_INFO:
        print '% Host: ' + module_settings[u'HOST']
    else:
        print '% Host: ' + module_settings[u'HOST']
        print '% Port: ' + module_settings[u'PORT']
        print '% Root: ' + module_settings[u'ROOT']
    
    if export_settings[u'EXPORT']:
        print '%'
        print '% Export format: ' + export_settings[u'FORMAT']
        print '% Export file: ' + export_settings[u'FILE']
        
    print '~~~~~~~~~~~~~~~~~~~~~~~~~~ ## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'

## ################################################################# ##

def __cli_print_end():
    print '~~~~~~~~~~~~~~~~~~~~~~~~~~ ## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'

## ################################################################# ##

def __cli_print_export(success, filename):
    if success:
        print 'Report successfully exported to ' + filename
    else:
        print 'Export of report failed (you may not have the appropriate rights).'

## ################################################################# ##

def __cli_print_update(dbname, success):
    if success:
        print 'Update of ' + dbname + ' database successful!'
    else:
        print 'Update of ' + dbname + ' database failed!'


## ################################################################# ##
## CLASS: ModuleRunner
## ################################################################# ##
## This class is in charge of running the attack modules as threads
## ################################################################# ##
    
class ModuleRunner(Thread):
    
    def __init__(self, module_settings, results, results_lock, switch, switch_lock):
        
        Thread.__init__(self)
        
        self.__module_settings = module_settings
        self.__results = results
        self.__results_lock = results_lock
        self.__switch = switch
        self.__switch_lock = switch_lock
        
    def run(self):
        
        cli = True
        
        module = self.__module_settings[u'MODULE']
        
        if module == MODULE_PSCAN:
            host = self.__module_settings[u'HOST']
            module_pscan.perform(host, cli, self.__results, self.__results_lock, self.__switch, self.__switch_lock)
        
        elif module == MODULE_INFO:
            host = self.__module_settings[u'HOST']
            module_info.perform(host, cli, self.__results, self.__results_lock, self.__switch, self.__switch_lock)
        
        elif module == MODULE_SPIDER:
            host = self.__module_settings[u'HOST']
            port = int(self.__module_settings[u'PORT'])
            root = self.__module_settings[u'ROOT']
            module_spider.perform(host, port, root, cli, self.__results, self.__results_lock, self.__switch, self.__switch_lock)
        
        elif module == MODULE_USCAN:
            hosts = [h.strip() for h in self.__module_settings[u'HOSTS'].split(u',')]
            ports = [int(p.strip()) for p in self.__module_settings[u'PORTS'].split(u',')]
            roots = [r.strip() for r in self.__module_settings[u'ROOTS'].split(u',')]
            skip = self.__module_settings[u'SKIP']
            server = self.__module_settings[u'SERVER']
            module_uscan.perform(hosts, ports, roots, server, skip, cli, self.__results, self.__results_lock, self.__switch, self.__switch_lock)
        
        elif module == MODULE_FUZZ:
            hosts = [h.strip() for h in self.__module_settings[u'HOSTS'].split(u',')]
            ports = [int(p.strip()) for p in self.__module_settings[u'PORTS'].split(u',')]
            roots = [r.strip() for r in self.__module_settings[u'ROOTS'].split(u',')]
            mode = self.__module_settings[u'MODE']
            
            if mode[0] == MODULE_FUZZ_GEN:
                generator = mode[1]
                module_fuzz.perform(hosts, ports, roots, 1, generator, cli, self.__results, self.__results_lock, self.__switch, self.__switch_lock)
            else:
                extension = mode[1]
                module_fuzz.perform(hosts, ports, roots, 0, extension, cli, self.__results, self.__results_lock, self.__switch, self.__switch_lock)
