# -*- coding: utf-8 -*-
## ################################################################# ##
## (C) SCRT - Information Security, 2007 - 2008 // author: ~SaD~     ##
## ################################################################# ##
## This program is free software: you can redistribute it and/or     ##
## modify it under the terms of the GNU General Public License as    ##
## published by the Free Software Foundation, either version 3 of    ##
## the License, or (at your option) any later version.               ##
## This program is distributed in the hope that it will be useful,   ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of    ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     ##
## GNU General Public License for more details.                      ##
## You should have received a copy of the GNU General Public License ##
## along with this program. If not, see http://www.gnu.org/licenses. ##
## ################################################################# ##
## last mod: 2008-02-20

from __future__ import with_statement
import wx
import threading
from sys import exit
from webshag.core import core_utilities, core_file, core_error
from webshag.modules import module_info, module_uscan, module_pscan, module_fuzz, module_spider
from webshag.export import export
from webshag.update import update
import gui_images

## ################################################################# ##
## MODULE CONSTANTS
## ################################################################# ##

WEBSHAG = u'webshag'
WEBSHAG_VERSION = u'1.00'

DEFAULT_HOST = u''
DEFAULT_SKIP = u''
DEFAULT_PORT = u'80'
DEFAULT_ROOT = u'/'
DEFAULT_EXTENSION = u''
DEFAULT_GENERATOR = u''

REFRESH_RATE = 500
SPLASH_TIME = 1500

MODULE_PSCAN = 'pscan'
MODULE_INFO = 'info'
MODULE_SPIDER = 'spider'
MODULE_USCAN = 'uscan'
MODULE_FUZZ = 'fuzz'

ABOUT_DESCRIPTION = u'"shagadelic" web server audit tool ;-)'
ABOUT_LICENSE = u"""
This program is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation, either version 3 of
the License, or (at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program. If not, see http://www.gnu.org/licenses.
"""
ABOUT_WEBSITE = u'http://www.scrt.ch'
ABOUT_COPY = u'(c) SCRT - Information Security, 2007-2008'
ABOUT_ME = u'Author: ~SaD~'

EXPORT_FORMATS = [u'xml', u'html', 'txt']
AUTO_WEB_SERVER = u'webserver (auto)'

TEXT_IDLE = u'< IDLE >'
TEXT_RUNNING = u'< RUNNING >'
TEXT_INVALID = u'< INVALID INPUT >'

## ################################################################# ##
## PUBLIC FUNCTIONS
## ################################################################# ##

def run():
    app = WebshagGUI()
    app.MainLoop()

## ################################################################# ##
## CLASS: WebshagGUI
## ################################################################# ##
## This class is in charge of launching the webshag GUI
## ################################################################# ##
class WebshagGUI(wx.App):
    
    def OnInit(self):
        splash = webshagSplash()
        splash.Show()
        window = WebshagWindow(None, -1, WEBSHAG + ' ' + WEBSHAG_VERSION)
        return True

## ################################################################# ##
## CLASS: WebshagSplash
## ################################################################# ##
## This class implements the splash screen
## ################################################################# ##
class webshagSplash(wx.SplashScreen):
    
    def __init__(self):
        splashImage = gui_images.getSplashBitmap()
        wx.SplashScreen.__init__(self, splashImage, wx.SPLASH_CENTRE_ON_SCREEN | wx.SPLASH_TIMEOUT, SPLASH_TIME, None)
        wx.Yield()

## ################################################################# ##
## CLASS: WebshagWindow
## ################################################################# ##
## This class implements the main window of the GUI
## ################################################################# ##
class WebshagWindow(wx.Frame):
    
    def __init__(self, parent, id, title):
        
        self.__frameWidth = wx.DisplaySize()[0] / 1.5
        self.__frameHeight = wx.DisplaySize()[1] / 1.5
        wx.Frame.__init__(self, parent, id, title, size=(self.__frameWidth, self.__frameHeight))

        # window icon
        icon = gui_images.getIcon()
        self.SetIcon(icon)
        
        # Variables ############################################
        self.__tool_running = {}
        self.__tool_running[MODULE_INFO] = False
        self.__tool_running[MODULE_USCAN] = False
        self.__tool_running[MODULE_FUZZ] = False
        self.__tool_running[MODULE_SPIDER] = False
        self.__tool_running[MODULE_PSCAN] = False
        self.__tool_results = {}
        self.__tool_results[MODULE_INFO] = []
        self.__tool_results[MODULE_USCAN] = []
        self.__tool_results[MODULE_FUZZ] = []
        self.__tool_results[MODULE_SPIDER] = []
        self.__tool_results[MODULE_PSCAN] = []
        self.__tool_lock = threading.RLock()
        
        # Widgets ############################################## 
        #
        # MENU BAR #
        self.__menuBar = wx.MenuBar()
        fileMenu = wx.Menu()
        self.__fileExportItem = wx.MenuItem(fileMenu, -1, u'&Export...')
        fileMenu.AppendItem(self.__fileExportItem)
        fileMenu.AppendSeparator()
        fileQuitItem = wx.MenuItem(fileMenu, -1, u'&Quit\tCtrl+Q')
        fileMenu.AppendItem(fileQuitItem)
        self.__menuBar.Append(fileMenu, u'&File')
        toolsMenu = wx.Menu()
        self.__toolsConfigItem = wx.MenuItem(toolsMenu, -1, u'&Config...')
        toolsMenu.AppendItem(self.__toolsConfigItem)
        updateMenu = wx.Menu()
        self.__updateNiktoItem = wx.MenuItem(updateMenu, -1, u'Nikto Database')
        updateMenu.AppendItem(self.__updateNiktoItem)
        self.__updateCustomItem = wx.MenuItem(updateMenu, -1, u'SCRT Database')
        updateMenu.AppendItem(self.__updateCustomItem)
        toolsMenu.AppendMenu(-1, '&Update', updateMenu)
        self.__menuBar.Append(toolsMenu, u'&Tools')
        helpMenu = wx.Menu()
        helpAboutItem = wx.MenuItem(helpMenu, -1, u'&About')
        helpMenu.AppendItem(helpAboutItem)
        self.__menuBar.Append(helpMenu, u'&Help')
        self.SetMenuBar(self.__menuBar)
        #
        # NOTEBOOK #
        self.__notebook = wx.Notebook(self, -1)
        self.__infoPanel = Info_Panel(self.__notebook, self)
        self.__openPortscanPanel = Port_Scanner_Panel(self.__notebook, self)
        self.__spiderPanel = Spider_Panel(self.__notebook, self)
        self.__urlScannerPanel = Url_Scanner_Panel(self.__notebook, self)
        self.__fuzzerPanel = Fuzzer_Panel(self.__notebook, self)
        self.__notebook.AddPage(self.__openPortscanPanel, 'PSCAN')
        self.__notebook.AddPage(self.__infoPanel, 'INFO')
        self.__notebook.AddPage(self.__spiderPanel, 'SPIDER')
        self.__notebook.AddPage(self.__urlScannerPanel, 'USCAN')
        self.__notebook.AddPage(self.__fuzzerPanel, 'FUZZ')
        
        # Event Bindings #######################################
        self.Bind(wx.EVT_MENU, self.__onQuit, id=fileQuitItem.GetId())
        self.Bind(wx.EVT_MENU, self.__onExport, id=self.__fileExportItem.GetId())
        self.Bind(wx.EVT_MENU, self.__onAbout, id=helpAboutItem.GetId())
        self.Bind(wx.EVT_MENU, self.__onConfig, id=self.__toolsConfigItem.GetId())
        self.Bind(wx.EVT_MENU, self.__onNiktoUpdate, id=self.__updateNiktoItem.GetId())
        self.Bind(wx.EVT_MENU, self.__onCustomUpdate, id=self.__updateCustomItem.GetId())
        self.Bind(wx.EVT_WINDOW_DESTROY, self.__onQuit)
        
        # Layout Management ####################################
        rootSizer = wx.BoxSizer(wx.VERTICAL)
        rootSizer.Add(self.__notebook, 1, wx.EXPAND)
        self.SetSizer(rootSizer)
        
        self.Center()
        self.Show(True)

    ## EVENT HANDLING METHODS ########################################## ##
    ## ################################################################# ##

    def __onQuit(self, event):
        self.Close(True)
        exit(0)
    
    def __onAbout(self, event):
        
        about = wx.AboutDialogInfo()
        about.SetIcon(gui_images.getSCRTIcon())
        about.SetWebSite(ABOUT_WEBSITE)
        about.SetCopyright(ABOUT_COPY)
        about.SetName(WEBSHAG)
        about.SetVersion(WEBSHAG_VERSION)
        about.SetDescription(ABOUT_DESCRIPTION)
        about.SetLicence(ABOUT_LICENSE)
        about.AddDeveloper(ABOUT_ME)
        about.AddDocWriter(ABOUT_ME)
        wx.AboutBox(about)
    
    def __onConfig(self, event):
        cfWindow = ConfigDialog(self, -1, u'Configuration')
        cfWindow.ShowModal()
        cfWindow.Destroy()
    
    def __onExport(self, event):
        results = {}
        with self.__tool_lock:
            for key in self.__tool_results.keys():
                if self.__tool_results[key] != []:
                    results[key] = self.__tool_results[key][:]
                else:
                    results[key] = None
        
        exWindow = ExportDialog(self, results)
        exWindow.ShowModal()
        exWindow.Destroy()
    
    def __onNiktoUpdate(self, event):
        success = update.update_nikto_database()
        if success:
            wx.MessageDialog(self, u'Update of Nikto database succeded!', style=wx.OK | wx.ICON_INFORMATION | wx.CENTRE).ShowModal()
        else:
            wx.MessageDialog(self, u'Update of Nikto database failed!', style=wx.OK | wx.ICON_ERROR | wx.CENTRE).ShowModal()
    
    def __onCustomUpdate(self, event):
        success = update.update_custom_database()
        if success:
            wx.MessageDialog(self, u'Update of SCRT database succeded!', style=wx.OK | wx.ICON_INFORMATION | wx.CENTRE).ShowModal()
        else:
            wx.MessageDialog(self, u'Update of SCRT database failed!', style=wx.OK | wx.ICON_ERROR | wx.CENTRE).ShowModal()
    
    
    ## COORDINATOR METHODS ############################################# ##
    ## ################################################################# ##
    ## The methods below are called by the tool panels. Indeed, the main
    ## window is in charge of coordinating the various tools (for
    ## instance preventing config file to be edited while some tool is
    ## running). For this reason, the instance of the main window is
    ## passed to tool panels as coordinator. Note that a dedicated class
    ## could be used as coordinator but there is no need for that.
    ## ################################################################# ##
    
    def __toolRunning(self):
        self.__toolsConfigItem.Enable(False)
        self.__updateNiktoItem.Enable(False)
        self.__updateCustomItem.Enable(False)
        self.__fileExportItem.Enable(False)
        self.__urlScannerPanel.enable(False)
        self.__fuzzerPanel.enable(False)

    def __noToolRunning(self):
        self.__toolsConfigItem.Enable(True)
        self.__updateNiktoItem.Enable(True)
        self.__updateCustomItem.Enable(True)
        self.__fileExportItem.Enable(True)
        self.__urlScannerPanel.enable(True)
        self.__fuzzerPanel.enable(True)
    
    def notify_start(self, tool):
        with self.__tool_lock:
            self.__tool_running[tool] = True
            self.__tool_results[tool] = []
            self.__toolRunning()
    
    def notify_stop(self, tool, results):
        with self.__tool_lock:
            self.__tool_running[tool] = False
            self.__tool_results[tool] = results
            if not True in self.__tool_running.values():
                self.__noToolRunning()

    def get_portscan_results(self):
        results = []
        with self.__tool_lock:
            for result in self.__tool_results[MODULE_PSCAN]:
                if result.has_key(u'PORTID'):
                    results.append(result[u'PORTID'])
        return results
    
    def get_info_results(self):
        results = []
        with self.__tool_lock:
            for result in self.__tool_results[MODULE_INFO]:
                if result.has_key(u'VHOST'):
                    results.append(result[u'VHOST'])
        return results
    
    def get_spider_results(self):
        results = []
        with self.__tool_lock:
            for result in self.__tool_results[MODULE_SPIDER]:
                if result.has_key(u'INTERNAL'):
                    results.append(result[u'INTERNAL'])
        return results


## ################################################################# ##
## CLASS: Port_Scanner_Panel
## ################################################################# ##
## This class provides the GUI for the port scanning module
## ################################################################# ##
class Port_Scanner_Panel(wx.Panel):
    
    
    def __init__(self, parent, coordinator):
        
        wx.Panel.__init__(self, parent)
        self.__coordinator = coordinator
        
        # Variables ############################################
        self.__tool = MODULE_PSCAN
        self.__results = []
        self.__openPorts = {}
        self.__switch = [False]
        self.__switch_lock = threading.RLock()
        self.__results_lock = threading.RLock()
        
        # Widgets ##############################################
        self.__hostText = wx.TextCtrl(self, -1, DEFAULT_HOST)
        self.__okButton = wx.Button(self, wx.ID_OK, u'&OK')
        self.__outputList = wx.ListBox(self, -1, style=wx.LB_SINGLE)
        self.__outputLogText = wx.TextCtrl(self, -1, '', style=wx.TE_MULTILINE | wx.TE_READONLY)
        self.__outputDetailsText = wx.TextCtrl(self, -1, '', style=wx.TE_MULTILINE | wx.TE_READONLY)
        self.__statusText = wx.StaticText(self, -1, TEXT_IDLE)
        self.__timer = wx.Timer(self, -1)
        
        # Event Bindings #######################################
        self.Bind(wx.EVT_BUTTON, self.__onOKButton, id=self.__okButton.GetId())
        self.Bind(wx.EVT_TIMER, self.__onTimer, id=self.__timer.GetId())
        self.Bind(wx.EVT_LISTBOX, self.__onSelect, id=self.__outputList.GetId())
        
        # Layout Management ####################################
        controlSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, 'Settings'), orient=wx.HORIZONTAL)
        controlSizerGrid = wx.GridBagSizer()
        controlSizerGrid.Add(wx.StaticText(self, -1, u'Target [host | IPv4]:'), (0, 0), (1, 2), wx.ALIGN_BOTTOM | wx.TOP, 5)
        controlSizerGrid.Add(self.__hostText, (1, 0), (1, 1), wx.EXPAND)
        controlSizerGrid.Add(self.__okButton, (1, 1), (1, 1), wx.EXPAND | wx.LEFT, 5)
        controlSizerGrid.AddGrowableCol(0)
        controlSizer.Add(controlSizerGrid, 1, wx.EXPAND)
        resultSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, 'Results'), orient=wx.VERTICAL)
        resultSizerGrid = wx.GridBagSizer()
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Open Ports:'), (0, 0), (1, 1), wx.LEFT | wx.ALIGN_BOTTOM, 5)
        resultSizerGrid.Add(self.__outputList, (1, 0), (3, 1), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Port Details:'), (0, 1), (1, 1), wx.LEFT | wx.ALIGN_BOTTOM, 5)
        resultSizerGrid.Add(self.__outputDetailsText, (1, 1), (3, 1), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Console:'), (4, 0), (1, 2), wx.LEFT | wx.ALIGN_BOTTOM, 5)
        resultSizerGrid.Add(self.__outputLogText, (5, 0), (1, 2), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.AddGrowableCol(1)
        resultSizerGrid.AddGrowableRow(1)
        resultSizerGrid.AddGrowableRow(2)
        resultSizerGrid.AddGrowableRow(3)
        resultSizerGrid.AddGrowableRow(5)
        resultSizer.Add(resultSizerGrid, 1, wx.EXPAND)
        rootSizer = wx.GridBagSizer()
        rootSizer.Add(controlSizer, (0, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(resultSizer, (1, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(self.__statusText, (2, 0), (1, 1), wx.ALIGN_CENTER |wx.ALL, 5)
        rootSizer.AddGrowableRow(1)
        rootSizer.AddGrowableCol(0)
        rootSizer.AddGrowableCol(1)
        self.SetSizer(rootSizer)
    
    ## EVENT HANDLING METHODS ########################################## ##
    ## ################################################################# ##
    
    def __onSelect(self, event):
        if len(self.__openPorts.keys()) > 0:
            index = self.__outputList.GetSelection()
            port = self.__outputList.GetString(index)
            self.__outputDetailsText.SetValue(self.__openPorts[port])
    
    def __onOKButton(self, event):
        self.__start()

    def __onTimer(self, event):
        with self.__switch_lock:
            if self.__switch[0]:
                pass
            else:
                self.__timer.Stop()
                self.__print_results()
                self.__okButton.Enable(True)
                with self.__results_lock:
                    self.__coordinator.notify_stop(self.__tool, self.__results)

    ## DISPLAY RELATED METHODS ######################################### ##
    ## ################################################################# ##

    def __print_start(self):
        self.__outputList.Clear()
        self.__outputLogText.Clear()
        self.__outputDetailsText.Clear()
        self.__statusText.SetLabel(TEXT_RUNNING)
        #self.Layout()
    
    def __print_invalid(self):
        self.__outputList.Clear()
        self.__outputLogText.Clear()
        self.__outputDetailsText.Clear()
        self.__statusText.SetLabel(TEXT_INVALID)
        #self.Layout()
    
    def __print_results(self):
        error = False
        with self.__results_lock:
            for result in self.__results:
    
                if result.has_key(u'ERROR'):
                    self.__outputLogText.AppendText( u'ERROR\t' + result[u'ERROR'] + u'\n')
                    error = True
                    
                elif result.has_key(u'TARGET'):
                    self.__outputLogText.AppendText( u'INFO\tScan of ' + result[u'TARGET'] + u' finished\n')
                    
                elif result.has_key(u'PORTID') and result.has_key(u'PROTOCOL'):
                   
                    details = u'PORT:\t\t' + result['PORTID'] + u' (' + result[u'PROTOCOL'].upper() + ')\n'
                    details += u''
                    
                    if result.has_key(u'SRV_NAME'):
                        details += 'SERVICE:\t\t' + result['SRV_NAME'] + u'\n'
                        
                    if result.has_key(u'SRV_PRODUCT'):
                        details += 'PRODUCT:\t' + result['SRV_PRODUCT'] + u'\n'
                    
                    if result.has_key(u'SRV_OS'):
                        details += 'SYSTEM:\t\t' + result['SRV_OS'] + u'\n'
                   
                    self.__openPorts[result[u'PORTID']] = details
        
        if not error:
            self.__outputLogText.AppendText( u'INFO\tFound ' + unicode(len(self.__openPorts)) + u' open ports.\n')
            
        self.__outputList.Set(sorted(self.__openPorts.keys()))
        self.__statusText.SetLabel(TEXT_IDLE)
        #self.Layout()

    ## MISC METHODS #################################################### ##
    ## ################################################################# ##

    def __start(self):
        with self.__switch_lock:
            if not self.__switch[0]:
                self.__load_entries()
                if self.__check_entries():
                    self.__coordinator.notify_start(self.__tool)
                    self.__switch[0] = True
                    self.__okButton.Enable(False)
                    self.__results = []
                    self.__openPorts = {}
                    self.__print_start()
                    Port_Scanner_Thread(self.__host, self.__results, self.__results_lock, self.callback_stop).start()
                    self.__timer.Start(REFRESH_RATE)
                else:
                    self.__print_invalid()
            else:
                pass
    
    def __stop(self):
        with self.__switch_lock:
            self.__switch[0] = False
    
    def __load_entries(self):
        self.__host = self.__hostText.GetValue().strip()
    
    def __check_entries(self):
        return core_utilities.check_host(self.__host)
    
    def callback_stop(self):
        self.__stop()


## ################################################################# ##
## CLASS: Port_Scanner_Thread
## ################################################################# ##
## This class implements the thread that is in charge of running
## the port scanning module
## ################################################################# ##
class Port_Scanner_Thread(threading.Thread):
    
    def __init__(self, host, results, results_lock, callback_stop):
        
        threading.Thread.__init__(self)
        
        self.__host = host
        self.__results = results
        self.__results_lock = results_lock
        self.__callback_stop = callback_stop
    
    def run(self):
        module_pscan.perform(self.__host, False, self.__results, self.__results_lock, [True], threading.Lock())
        self.__callback_stop()


## ################################################################# ##
## CLASS: Info_Panel
## ################################################################# ##
## This class provides the GUI for the info module
## ################################################################# ##
class Info_Panel(wx.Panel):
    
    def __init__(self, parent, coordinator):
    
        wx.Panel.__init__(self, parent)
        self.__coordinator = coordinator
        
        # Variables ############################################
        self.__tool = MODULE_INFO
        self.__results = []
        self.__domains = []
        self.__switch = [False]
        self.__switch_lock = threading.RLock()
        self.__results_lock = threading.RLock()
        
        # Widgets ##############################################
        self.__hostText = wx.TextCtrl(self, -1, DEFAULT_HOST)
        self.__okButton = wx.Button(self, wx.ID_OK, u'&OK')
        self.__outputList = wx.ListBox(self, -1, style=wx.LB_SINGLE)
        self.__outputLogText = wx.TextCtrl(self, -1, '', style=wx.TE_MULTILINE | wx.TE_READONLY) 
        self.__outputCopyText = wx.TextCtrl(self, -1, '', style=wx.TE_READONLY)
        self.__statusText = wx.StaticText(self, -1, TEXT_IDLE)
        self.__timer = wx.Timer(self, -1)
        
        # Event Bindings ################################################################
        self.Bind(wx.EVT_BUTTON, self.__onOKButton, id=self.__okButton.GetId())
        self.Bind(wx.EVT_TIMER, self.__onTimer, id=self.__timer.GetId())
        self.Bind(wx.EVT_LISTBOX, self.__onSelect, id=self.__outputList.GetId())
        
        # Layout Management #############################################################
        controlSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, 'Settings'), orient=wx.HORIZONTAL)
        controlSizerGrid = wx.GridBagSizer()
        controlSizerGrid.Add(wx.StaticText(self, -1, u'Target [host | IPv4]:'), (0, 0), (1, 2), wx.ALIGN_BOTTOM | wx.TOP, 5)
        controlSizerGrid.Add(self.__hostText, (1, 0), (1, 1), wx.EXPAND)
        controlSizerGrid.Add(self.__okButton, (1, 1), (1, 1), wx.EXPAND | wx.LEFT, 5)
        controlSizerGrid.AddGrowableCol(0)
        controlSizer.Add(controlSizerGrid, 1, wx.EXPAND)
        resultSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, 'Results'), orient=wx.VERTICAL)
        resultSizerGrid = wx.GridBagSizer()
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Domains:'), (0, 0), (1, 1), wx.LEFT | wx.ALIGN_BOTTOM, 5)
        resultSizerGrid.Add(self.__outputList, (1, 0), (3, 1), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Copy:'), (4, 0), (1, 1), wx.LEFT | wx.ALIGN_BOTTOM, 5)
        resultSizerGrid.Add(self.__outputCopyText, (5, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Console:'), (6, 0), (1, 1), wx.LEFT | wx.ALIGN_BOTTOM, 5)
        resultSizerGrid.Add(self.__outputLogText, (7, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.AddGrowableCol(0)
        resultSizerGrid.AddGrowableRow(1)
        resultSizerGrid.AddGrowableRow(2)
        resultSizerGrid.AddGrowableRow(3)
        resultSizerGrid.AddGrowableRow(7)
        resultSizer.Add(resultSizerGrid, 1, wx.EXPAND)
        rootSizer = wx.GridBagSizer()
        rootSizer.Add(controlSizer, (0, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(resultSizer, (1, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(self.__statusText, (2, 0), (1, 1), wx.ALIGN_CENTER |wx.ALL, 5)
        rootSizer.AddGrowableRow(1)
        rootSizer.AddGrowableCol(0)
        rootSizer.AddGrowableCol(1)
        self.SetSizer(rootSizer)
    
    ## EVENT HANDLING METHODS ########################################## ##
    ## ################################################################# ##
    
    def __onSelect(self, event):
        if len(self.__domains) > 0:
            index = self.__outputList.GetSelection()
            domain = self.__outputList.GetString(index)
            self.__outputCopyText.SetValue(domain)
    
    def __onOKButton(self, event):
        self.__start()
    
    def __onTimer(self, event):
        with self.__switch_lock:
            if self.__switch[0]:
                pass
            else:
                self.__timer.Stop()
                self.__print_results()
                self.__okButton.Enable(True)
                with self.__results_lock:
                    self.__coordinator.notify_stop(self.__tool, self.__results)
    
    ## DISPLAY RELATED METHODS ######################################### ##
    ## ################################################################# ##
    
    def __print_start(self):
        self.__outputList.Clear()
        self.__outputLogText.Clear()
        self.__outputCopyText.Clear()
        self.__statusText.SetLabel(TEXT_RUNNING)
    
    def __print_invalid(self):
        self.__outputList.Clear()
        self.__outputLogText.Clear()
        self.__outputCopyText.Clear()
        self.__statusText.SetLabel(TEXT_INVALID)
    
    def __print_results(self):
        error = False
        
        with self.__results_lock:
            for result in self.__results:
                if result.has_key(u'ERROR'):
                    self.__outputLogText.AppendText( u'ERROR\t' + result[u'ERROR'] + u'\n')
                    error = True
                    
                elif result.has_key(u'TARGET'):
                    self.__outputLogText.AppendText( u'INFO\tDomains of ' + result[u'TARGET'] + u' retrieved\n')
                
                else:
                    self.__domains.append(result[u'VHOST'])
        if not error:
            self.__outputLogText.AppendText( u'INFO\tFound ' + unicode(len(self.__domains)) + u' domains.\n')
        self.__outputList.Set(self.__domains)
        self.__statusText.SetLabel(TEXT_IDLE)
    
    ## MISC METHODS #################################################### ##
    ## ################################################################# ##
    
    def __start(self):
        with self.__switch_lock:
            if not self.__switch[0]:
                self.__load_entries()
                if self.__check_entries():
                    self.__coordinator.notify_start(self.__tool)
                    self.__switch[0] = True
                    self.__okButton.Enable(False)
                    self.__results = []
                    self.__domains = []
                    self.__print_start()
                    Info_Thread(self.__host, self.__results, self.__results_lock, self.__switch, self.__switch_lock, self.callback_stop).start()
                    self.__timer.Start(REFRESH_RATE)
                else:
                    self.__print_invalid()
            else:
                pass
    
    def __stop(self):
        with self.__switch_lock:
            self.__switch[0] = False
    
    def __load_entries(self):
        self.__host = self.__hostText.GetValue().strip()
    
    def __check_entries(self):
        return core_utilities.check_host(self.__host)
    
    def callback_stop(self):
        self.__stop()

## ################################################################# ##
## CLASS: Info_Thread
## ################################################################# ##
## This class implements the thread that is in charge of running
## the info module
## ################################################################# ##
class Info_Thread(threading.Thread):
    
    def __init__(self, host, results, results_lock, switch, switch_lock, callback_stop):
        threading.Thread.__init__(self)
        self.__host = host
        self.__results = results
        self.__results_lock = results_lock
        self.__switch = switch
        self.__switch_lock = switch_lock
        self.__callback_stop = callback_stop
    
    def run(self):
        module_info.perform(self.__host, False, self.__results, self.__results_lock, self.__switch, self.__switch_lock)
        self.__callback_stop()


## ################################################################# ##
## CLASS: Spider_Panel
## ################################################################# ##
## This class provides the GUI for the spider module
## ################################################################# ##
class Spider_Panel(wx.Panel):
    
    def __init__(self, parent, coordinator):
        
        wx.Panel.__init__(self, parent)
        self.__coordinator = coordinator
        
        # Variables #############################################
        self.__tool = MODULE_SPIDER
        self.__switch = [False]
        self.__switch_lock = threading.RLock()
        self.__results = []
        self.__displayedResults = []
        self.__results_lock = threading.RLock()
        self.__timer = wx.Timer(self, -1)
        
        # Widgets #############################################
        self.__hostText = wx.TextCtrl(self, -1, DEFAULT_HOST)
        self.__portText = wx.TextCtrl(self, -1, DEFAULT_PORT)
        self.__rootText = wx.TextCtrl(self, -1, DEFAULT_ROOT)
        self.__okButton = wx.Button(self, wx.ID_OK, u'&OK')
        self.__stopButton = wx.Button(self, wx.ID_STOP, u'&Stop')
        self.__internalList = wx.ListBox(self, -1, style=wx.LB_SINGLE)
        self.__externalList = wx.ListBox(self, -1, style=wx.LB_SINGLE)
        self.__emailList = wx.ListBox(self, -1, style=wx.LB_SINGLE)
        self.__outputLogText = wx.TextCtrl(self, -1, '', style=wx.TE_MULTILINE | wx.TE_READONLY) 
        self.__statusText = wx.StaticText(self, -1, TEXT_IDLE)
        
        # Event Bindings #############################################
        self.Bind(wx.EVT_BUTTON, self.__onOKButton, id=self.__okButton.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onStopButton, id=self.__stopButton.GetId())
        self.Bind(wx.EVT_TIMER, self.__onTimer, id=self.__timer.GetId())
        
        # Layout Management #############################################
        rootSizer = wx.GridBagSizer()
        settingsSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, 'Settings'), orient=wx.VERTICAL)
        resultSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, 'Results'), orient=wx.VERTICAL)
        settingsSizerGrid = wx.GridBagSizer()
        resultSizerGrid = wx.GridBagSizer()
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'Target [host | IPv4]:'), (0, 0), (1, 1), wx.ALIGN_BOTTOM)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'Port [80]:'), (0, 1), (1, 1), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'Start [/index.html]:'), (0, 2), (1, 1), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u''), (0, 3), (1, 2), wx.EXPAND)
        settingsSizerGrid.Add(self.__hostText, (1, 0), (1, 1), wx.EXPAND)
        settingsSizerGrid.Add(self.__portText, (1, 1), (1, 1), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__rootText, (1, 2), (1, 1), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__okButton, (1, 3), (1, 1), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__stopButton, (1, 4), (1, 1), wx.EXPAND)
        settingsSizerGrid.AddGrowableCol(0)
        settingsSizerGrid.AddGrowableCol(2)
        
        resultSizerGrid.Add(wx.StaticText(self, -1, u'internal directories:'), (0, 0), (1, 1), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        resultSizerGrid.Add(self.__internalList, (1, 0), (5, 1), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.Add(wx.StaticText(self, -1, u'emails:'), (0, 1), (1, 1), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        resultSizerGrid.Add(self.__emailList, (1, 1), (2, 1), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.Add(wx.StaticText(self, -1, u'external links:'), (3, 1), (1, 1), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        resultSizerGrid.Add(self.__externalList, (4, 1), (2, 1), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Console:'), (6, 0), (1, 2), wx.LEFT | wx.ALIGN_BOTTOM, 5)
        resultSizerGrid.Add(self.__outputLogText, (7, 0), (1, 2), wx.EXPAND | wx.ALL, 5)
        
        resultSizerGrid.AddGrowableCol(0)
        resultSizerGrid.AddGrowableCol(1)
        resultSizerGrid.AddGrowableRow(1)
        resultSizerGrid.AddGrowableRow(2)
        resultSizerGrid.AddGrowableRow(4)
        resultSizerGrid.AddGrowableRow(5)
        resultSizerGrid.AddGrowableRow(7)
        
        settingsSizer.Add(settingsSizerGrid, 1, wx.EXPAND)
        resultSizer.Add(resultSizerGrid, 1, wx.EXPAND)
        rootSizer.Add(settingsSizer, (0, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(resultSizer, (1, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(self.__statusText, (2, 0), (1, 1), wx.ALIGN_CENTER |wx.ALL, 5)
        rootSizer.AddGrowableRow(1)
        rootSizer.AddGrowableCol(0)
        self.SetSizer(rootSizer)
    
    ## EVENT HANDLING METHODS ########################################## ##
    ## ################################################################# ##
    
    def __onStopButton(self, event):
        self.__end_scan()
        
    def __onOKButton(self, event):
        self.__start_scan()
    
    def __onTimer(self, event):
        with self.__switch_lock:
            if self.__switch[0]:
                self.__print_results(False)
            else:
                self.__print_results(True)
                self.__timer.Stop()
                self.__okButton.Enable(True)
                with self.__results_lock:
                    self.__coordinator.notify_stop(self.__tool, self.__displayedResults)
    
    ## DISPLAY RELATED METHODS ######################################### ##
    ## ################################################################# ##
    
    def __print_start(self):
        self.__internalList.Clear()
        self.__externalList.Clear()
        self.__emailList.Clear()
        self.__outputLogText.Clear()
        self.__statusText.SetLabel(TEXT_RUNNING)

    def __print_invalid(self):
        self.__internalList.Clear()
        self.__externalList.Clear()
        self.__emailList.Clear()
        self.__outputLogText.Clear()
        self.__statusText.SetLabel(TEXT_INVALID)

    def __print_results(self, finished):
        tmpRes = []
        
        with self.__results_lock:
            
            while len(self.__results) > 0:
                rs = self.__results.pop(0)
                tmpRes.append(rs)
                self.__displayedResults.append(rs)
                
        for result in tmpRes:
            
            if result.has_key(u'ERROR'):
                self.__outputLogText.AppendText( u'ERROR\t' + result[u'ERROR'] + u'\n')
                
            elif result.has_key(u'TARGET'):
                self.__outputLogText.AppendText( u'INFO\tSpidering ' + result[u'TARGET'] + u'\n')
               
            elif result.has_key(u'INTERNAL'):
                self.__internalList.AppendAndEnsureVisible(result[u'INTERNAL'])
                
            elif result.has_key(u'EXTERNAL'):
                self.__externalList.AppendAndEnsureVisible(result[u'EXTERNAL'])
                
            elif result.has_key(u'EMAIL'):
                self.__emailList.AppendAndEnsureVisible(result[u'EMAIL'])
                
        if finished:
            self.__statusText.SetLabel(TEXT_IDLE)
    
    ## MISC METHODS #################################################### ##
    ## ################################################################# ##
    
    def __start_scan(self):
        with self.__switch_lock:
            if not self.__switch[0]:
                self.__load_entries()
                if self.__check_entries():
                    self.__coordinator.notify_start(self.__tool)
                    self.__switch[0] = True
                    self.__okButton.Enable(False)
                    self.__results = []
                    self.__internal = []
                    self.__external = []
                    self.__email = []
                    self.__displayedResults = []
                    self.__print_start()
                    st = Spider_Thread(self.__host, self.__port, self.__root, self.__results, self.__results_lock, self.__switch, self.__switch_lock, self.callback_end)
                    st.start()
                    self.__timer.Start(REFRESH_RATE*2)
                else:
                    self.__print_invalid()
            else:
                pass
    
    def __end_scan(self):
        with self.__switch_lock:
            self.__switch[0] = False
    
    def __load_entries(self):
        self.__host = self.__hostText.GetValue().strip()
        self.__port = self.__portText.GetValue().strip()
        self.__root = self.__rootText.GetValue().strip()
    
    def __check_entries(self):
        return core_utilities.check_host(self.__host) and core_utilities.check_port_string(self.__port) and core_utilities.check_spider_root(self.__root)
    
    def callback_end(self):
        self.__end_scan()
        

## ################################################################# ##
## CLASS: Spider_Thread
## ################################################################# ##
## This class implements the thread that is in charge of running
## the spider module
## ################################################################# ##
class Spider_Thread(threading.Thread):
    
    def __init__(self, host, port, root, results, results_lock, switch, switch_lock, cb_end):
        threading.Thread.__init__(self)
        self.__host = host
        self.__port = port
        self.__root = root
        self.__results = results
        self.__results_lock = results_lock
        self.__switch = switch
        self.__switch_lock = switch_lock
        self.__cb_end = cb_end
    
    def run(self):
        module_spider.perform(self.__host, int(self.__port), self.__root, False, self.__results, self.__results_lock, self.__switch, self.__switch_lock)
        self.__cb_end()



## ################################################################# ##
## CLASS: Url_Scanner_Panel
## ################################################################# ##
## This class provides the GUI for the URL Scanner module
## ################################################################# ##
class Url_Scanner_Panel(wx.Panel):
    
    def __init__(self, parent, coordinator):
        
        wx.Panel.__init__(self, parent)
        self.__coordinator = coordinator
        
        # Variables #############################################
        self.__tool = MODULE_USCAN
        self.__switch = [False]
        self.__results = []
        self.__targets = {}
        self.__displayedResults = []
        self.__switch_lock = threading.RLock()
        self.__results_lock = threading.RLock()
        
        self.__currentTarget = ''
        
        # Widgets ##############################################
        self.__timer = wx.Timer(self, -1)
        self.__hostsText = wx.TextCtrl(self, -1, DEFAULT_HOST)
        self.__portsText = wx.TextCtrl(self, -1, DEFAULT_PORT)
        self.__rootsText = wx.TextCtrl(self, -1, DEFAULT_ROOT)
        self.__skipText = wx.TextCtrl(self, -1, DEFAULT_SKIP)
        self.__server_choices = [AUTO_WEB_SERVER]
        
        try:
            tmp_choices = []
            tmp_banners = core_file.db_load_known_banners()
            for tmp in tmp_banners.values():
                if tmp not in tmp_choices:
                    tmp_choices.append(tmp)
            tmp_choices.sort()
            self.__server_choices.extend(tmp_choices)
        except core_error.Config_Error, e:
            pass
        
        self.__serverCombo = wx.ComboBox(self, -1, style=wx.CB_READONLY, choices=self.__server_choices)
        self.__serverCombo.SetValue(self.__server_choices[0])
        self.__importButton = wx.Button(self, -1, u'Import...')
        self.__okButton = wx.Button(self, wx.ID_OK, u'&OK')
        self.__stopButton = wx.Button(self, wx.ID_STOP, u'&Stop')
        
        self.__outputTargetList = wx.ListBox(self, -1, style=wx.LB_SINGLE)
        self.__outputTargetText = wx.TextCtrl(self, -1, '', style=wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_RICH2)
        self.__outputLogText = wx.TextCtrl(self, -1, '', style=wx.TE_MULTILINE | wx.TE_READONLY)
        self.__statusText = wx.StaticText(self, -1, TEXT_IDLE)
        
        # Event Bindings ################################################################
        self.Bind(wx.EVT_BUTTON, self.__onOKButton, id=self.__okButton.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onStopButton, id=self.__stopButton.GetId())
        self.Bind(wx.EVT_TIMER, self.__onTimer, id=self.__timer.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onImport, id=self.__importButton.GetId())
        self.Bind(wx.EVT_LISTBOX, self.__onTargetSelect, id=self.__outputTargetList.GetId())
        
        # Layout Management #############################################################
        settingsSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, 'Settings'), orient=wx.VERTICAL)
        settingsSizerGrid = wx.GridBagSizer()
        settingsSizerGrid.AddGrowableCol(0)
        settingsSizerGrid.AddGrowableCol(1)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'Target(s) [host1, host2,...]:'), (0, 0), (1, 2), wx.ALIGN_BOTTOM | wx.TOP, 5)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'Port(s) [80, 8080,...]:'), (0, 2), (1, 1), wx.ALIGN_BOTTOM | wx.TOP | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__hostsText, (1, 0), (1, 2), wx.EXPAND)
        settingsSizerGrid.Add(self.__portsText, (1, 2), (1, 1), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__okButton, (1, 3), (1, 1), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__stopButton, (1, 4), (1, 1), wx.EXPAND)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'Root directoties [/, /dir/,...]:'), (2, 0), (1, 1), wx.ALIGN_BOTTOM)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'Skip String [Not Found]:'), (2, 1), (1, 2), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__importButton, (2, 3), (1, 2), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__rootsText, (3, 0), (1, 1), wx.EXPAND)
        settingsSizerGrid.Add(self.__skipText, (3, 1), (1, 2), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__serverCombo, (3, 3), (1, 2), wx.EXPAND | wx.LEFT, 5)
        settingsSizer.Add(settingsSizerGrid, 1, wx.EXPAND)
       
        resultSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, 'Results'), orient=wx.VERTICAL)
        resultSizerGrid = wx.GridBagSizer()
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Targets:'), (0, 0), (1, 1), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        resultSizerGrid.Add(self.__outputTargetList, (1, 0), (3, 1), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Results:'), (0, 1), (1, 4), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        resultSizerGrid.Add(self.__outputTargetText, (1, 1), (3, 4), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Console:'), (4, 0), (1, 5), wx.LEFT | wx.ALIGN_BOTTOM, 5)
        resultSizerGrid.Add(self.__outputLogText, (5, 0), (1, 5), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.AddGrowableCol(0)
        resultSizerGrid.AddGrowableCol(1)
        resultSizerGrid.AddGrowableCol(2)
        resultSizerGrid.AddGrowableCol(3)
        resultSizerGrid.AddGrowableCol(4)
        resultSizerGrid.AddGrowableRow(1)
        resultSizerGrid.AddGrowableRow(2)
        resultSizerGrid.AddGrowableRow(3)
        resultSizerGrid.AddGrowableRow(5)
        
        resultSizer.Add(resultSizerGrid, 1, wx.EXPAND)
        rootSizer = wx.GridBagSizer()
        rootSizer.Add(settingsSizer, (0, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(resultSizer, (1, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(self.__statusText, (2, 0), (1, 1), wx.ALIGN_CENTER |wx.ALL, 5)
        rootSizer.AddGrowableRow(1)
        rootSizer.AddGrowableCol(0)
        self.SetSizer(rootSizer)
    
    ## EVENT HANDLING METHODS ########################################## ##
    ## ################################################################# ##
    
    def __onTargetSelect(self, event):
        tgt = self.__outputTargetList.GetString(self.__outputTargetList.GetSelection())
        if self.__targets.has_key(tgt):
            self.__outputTargetText.SetValue(self.__targets[tgt])
        else:
            self.__outputTargetText.Clear()
    
    def __onTimer(self, event):
        with self.__switch_lock:
            if self.__switch[0]:
                self.__print_results(False)
            else:
                self.__timer.Stop()
                self.__print_results(True)
                self.__okButton.Enable(True)
                self.__outputTargetList.Enable(True)
                self.__coordinator.notify_stop(self.__tool, self.__displayedResults)
    
    def __onStopButton(self, event):
        self.__end_scan()

    def __onOKButton(self, event):
        self.__start_scan()

    def __onImport(self, event):
        iWindow = ImportDialog(self, self.__coordinator, self.callback_import)
        iWindow.ShowModal()
        iWindow.Destroy()
    
    ## DISPLAY RELATED METHODS ######################################### ##
    ## ################################################################# ##
    
    def __print_start(self):
        self.__outputTargetList.Clear()
        self.__outputTargetText.Clear()
        self.__outputLogText.Clear()
        self.__statusText.SetLabel(TEXT_RUNNING)
        #self.Layout()
    
    def __print_invalid(self):
        self.__outputTargetList.Clear()
        self.__outputTargetText.Clear()
        self.__outputLogText.Clear()
        self.__statusText.SetLabel(TEXT_INVALID)
        #self.Layout()
    
    def __print_results(self, finished):
        
        tmpRes = []
        
        with self.__results_lock:
            while len(self.__results) > 0:
                rs = self.__results.pop(0)
                tmpRes.append(rs)
                self.__displayedResults.append(rs)
                
        for result in tmpRes:
                
            if result.has_key(u'ERROR'):
                if result.has_key(u'HOST') and result.has_key(u'PORT'):
                    tgt = result[u'HOST'] + u':' + result[u'PORT']
                    if self.__targets.has_key(tgt):
                        self.__targets[tgt] += u'\n[ERR]\t' + result[u'ERROR'] + u'\n'
                        if self.__currentTarget == tgt:
                            self.__outputTargetText.AppendText(u'\n[ERR]\t' + result[u'ERROR'] + u'\n')
                            self.__outputTargetText.ShowPosition(self.__outputTargetText.GetLastPosition())
                    else:
                        self.__outputLogText.AppendText( u'ERROR\t' + result[u'ERROR'] + u'\n')
                else:
                    self.__outputLogText.AppendText(u'ERROR\t' + result[u'ERROR'] + u'\n')
            
            elif result.has_key(u'TARGET'):
                tgt = result[u'HOST'] + u':' + result[u'PORT']
                # add new target #
                self.__targets[tgt] = u''
                self.__currentTarget = tgt
                self.__outputTargetText.Clear()
                self.__outputLogText.AppendText( u'TARGET\tScanning ' + result[u'TARGET'] + u'\n')
                self.__outputTargetList.AppendAndEnsureVisible(tgt)
            
            elif result.has_key(u'INFO'):
                tgt = result[u'HOST'] + u':' + result[u'PORT']
                if self.__targets.has_key(tgt):
                    self.__targets[tgt] += u'\n[INF]\t' + result[u'INFO'] + u'\n'
                    if self.__currentTarget == tgt:
                        self.__outputTargetText.AppendText(u'\n[INF]\t' + result[u'INFO'] + u'\n')
                        self.__outputTargetText.ShowPosition(self.__outputTargetText.GetLastPosition())
                else:
                    self.__outputLogText.AppendText(u'INFO\t' + result[u'INFO'] + u'\n')
            
            elif result.has_key(u'BANNER'):
                tgt = result[u'HOST'] + u':' + result[u'PORT']
                if self.__targets.has_key(tgt):
                    self.__targets[tgt] += u'\n[SRV]\t' + result[u'BANNER'] + u' ==> ' + result[u'SERVER'] + u'\n'
                    if self.__currentTarget == tgt:
                        self.__outputTargetText.AppendText(u'\n[SRV]\t' + result[u'BANNER'] + u' ==> ' + result[u'SERVER'] + u'\n')
                        self.__outputTargetText.ShowPosition(self.__outputTargetText.GetLastPosition())
                else:
                    self.__outputLogText.AppendText(u'SERVER\tServer Banner (' + result[u'HOST'] + u' : ' +result[u'PORT'] + u'): ' + result[u'BANNER'] + u' (' + result[u'SERVER'] + u')\n')
            
            else:
                tgt = result[u'HOST'] + u':' + result[u'PORT']
                text = u'\n[' + result[u'CODE'] + u']\t' + result[u'PATH'] + u'\n' + result[u'DESCRIPTION'] + u'\n'
                if self.__targets.has_key(tgt):
                    self.__targets[tgt] += text
                    if self.__currentTarget == tgt:
                        self.__outputTargetText.AppendText(text)
                        self.__outputTargetText.ShowPosition(self.__outputTargetText.GetLastPosition())
        
        if finished:
            self.__statusText.SetLabel(TEXT_IDLE)
            #self.Layout()
    
    ## MISC METHODS #################################################### ##
    ## ################################################################# ##
    
    def __start_scan(self):
        with self.__switch_lock:
            if not self.__switch[0]:
                self.__load_entries()
                if self.__check_entries():
                    self.__switch[0] = True
                    self.__coordinator.notify_start(self.__tool)
                    self.__outputTargetList.Enable(False)
                    self.__okButton.Enable(False)
                    self.__results = []
                    self.__targets = {}
                    self.__currentTarget = u''
                    self.__displayedResults = []
                    self.__print_start()
                    thread = Url_Scanner_Thread(self.__hosts, self.__ports, self.__roots, self.__server, self.__skip, self.__results, self.__results_lock, self.__switch, self.__switch_lock, self.callback_end)
                    thread.start()
                    self.__timer.Start(REFRESH_RATE)
                else:
                    self.__print_invalid()
            else:
                pass
    
    def __end_scan(self):
        with self.__switch_lock:
            self.__switch[0] = False
    
    def __load_entries(self):
        self.__hosts = self.__hostsText.GetValue().strip()
        self.__ports = self.__portsText.GetValue().strip()
        self.__roots = self.__rootsText.GetValue().strip()
        if self.__serverCombo.GetValue() != AUTO_WEB_SERVER:
            self.__server = self.__serverCombo.GetValue()
        else:
            self.__server = u''
        self.__skip = self.__skipText.GetValue().strip()
    
    def __check_entries(self):
        return core_utilities.check_host_list(self.__hosts) and core_utilities.check_port_list(self.__ports) and core_utilities.check_root_list(self.__roots) and core_utilities.check_generic_string(self.__skip)

    def enable(self, state):
        self.__importButton.Enable(state)

    def callback_end(self):
        self.__end_scan()
    
    def callback_import(self, hosts, ports, roots):
        if hosts != u'':
            self.__hostsText.SetValue(hosts)
        if ports != u'':
            self.__portsText.SetValue(ports)
        if roots != u'':
            self.__rootsText.SetValue(roots)
    

## ################################################################# ##
## CLASS: Url_Scanner_Thread
## ################################################################# ##
## This class implements the thread that is in charge of running
## the URL scanner module
## ################################################################# ##
class Url_Scanner_Thread(threading.Thread):
    
    def __init__(self, hosts, ports, roots, server, skip, results, results_lock, switch, switch_lock, cb_end):
        
        threading.Thread.__init__(self)
        self.__hosts = hosts
        self.__ports = ports
        self.__roots = roots
        self.__server = server
        self.__skip = skip
        self.__results = results
        self.__results_lock = results_lock
        self.__switch = switch
        self.__switch_lock = switch_lock
        self.__cb_end = cb_end
    
    def run(self):
        
        hosts = [h.strip() for h in self.__hosts.split(u',')]
        ports = [int(p.strip()) for p in self.__ports.split(u',')]
        roots = [r.strip() for r in self.__roots.split(u',')]
        results = module_uscan.perform(hosts, ports, roots, self.__server, self.__skip, False, self.__results, self.__results_lock, self.__switch, self.__switch_lock)
        self.__cb_end()

## ################################################################# ##
## CLASS: Fuzzer_Panel
## ################################################################# ##
## This class provides the GUI for the Fuzzer module
## ################################################################# ##
class Fuzzer_Panel(wx.Panel):
    
    def __init__(self, parent, coordinator):
        
        wx.Panel.__init__(self, parent)
        self.__coordinator = coordinator
        
        # Variables #############################################
        self.__tool = MODULE_FUZZ
        self.__switch = [False]
        self.__switch_lock = threading.RLock()
        self.__results = []
        self.__targets = {}
        self.__displayedResults = []
        self.__results_lock = threading.Lock()
        self.__timer = wx.Timer(self, -1)
        self.__currentTarget = ''
        
        # Widgets #############################################
        self.__hostsText = wx.TextCtrl(self, -1, DEFAULT_HOST)
        self.__portsText = wx.TextCtrl(self, -1, DEFAULT_PORT)
        self.__rootsText = wx.TextCtrl(self, -1, DEFAULT_ROOT)
        self.__extensionText = wx.TextCtrl(self, -1, DEFAULT_EXTENSION)
        self.__generatorText = wx.TextCtrl(self, -1, DEFAULT_GENERATOR)
        self.__modeToggle = wx.ToggleButton(self, -1, '', style=wx.RB_GROUP)
        self.__importButton = wx.Button(self, -1, u'Import...')
        self.__okButton = wx.Button(self, wx.ID_OK, u'&OK')
        self.__stopButton = wx.Button(self, wx.ID_STOP, u'&Stop')
        
        self.__outputTargetList = wx.ListBox(self, -1, style=wx.LB_SINGLE)
        self.__outputTargetText = wx.TextCtrl(self, -1, '', style=wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_RICH2)
        self.__outputLogText = wx.TextCtrl(self, -1, '', style=wx.TE_MULTILINE | wx.TE_READONLY)
        self.__statusText = wx.StaticText(self, -1, TEXT_IDLE)
        
        self.__mode = 0 # listmode
        self.__modeToggle.SetValue(False)
        self.__extensionText.Enable(True)
        self.__generatorText.Enable(False)
        self.__modeToggle.SetLabel(u'Switch to Gen Mode')
        
        # Event Bindings #############################################
        self.Bind(wx.EVT_BUTTON, self.__onOKButton, id=self.__okButton.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onStopButton, id=self.__stopButton.GetId())
        self.Bind(wx.EVT_TIMER, self.__onTimer, id=self.__timer.GetId())
        self.Bind(wx.EVT_TOGGLEBUTTON, self.__onToggle, id=self.__modeToggle.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onImport, id=self.__importButton.GetId())
        self.Bind(wx.EVT_LISTBOX, self.__onTargetSelect, id=self.__outputTargetList.GetId())
        
        # Layout Management #############################################
        rootSizer = wx.GridBagSizer()
        settingsSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, 'Settings'), orient=wx.VERTICAL)
        
        settingsSizerGrid = wx.GridBagSizer()
        settingsSizerGrid.AddGrowableCol(0)
        settingsSizerGrid.AddGrowableCol(1)
        settingsSizer.Add(settingsSizerGrid, 1, wx.EXPAND)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'Target(s) [host1, host2,...]:'), (0, 0), (1, 2), wx.ALIGN_BOTTOM | wx.TOP, 5)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'Port(s) [80, 8080,...]:'), (0, 2), (1, 1), wx.ALIGN_BOTTOM | wx.TOP | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__hostsText, (1, 0), (1, 2), wx.EXPAND)
        settingsSizerGrid.Add(self.__portsText, (1, 2), (1, 1), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__okButton, (1, 3), (1, 1), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__stopButton, (1, 4), (1, 1), wx.EXPAND)
        settingsSizerGrid.Add(self.__importButton, (2, 3), (1, 2), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'Root directoties [/, /dir/,...]:'), (2, 0), (1, 1), wx.ALIGN_BOTTOM)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'Expression [log_[a-z]{1}[0-9]{2}.txt]:'), (2, 1), (1, 1), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        settingsSizerGrid.Add(wx.StaticText(self, -1, u'File Extension [.html]:'), (2, 2), (1, 1), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__rootsText, (3, 0), (1, 1), wx.EXPAND)
        settingsSizerGrid.Add(self.__generatorText, (3, 1), (1, 1), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__extensionText, (3, 2), (1, 1), wx.EXPAND | wx.LEFT, 5)
        settingsSizerGrid.Add(self.__modeToggle, (3, 3), (1, 2), wx.EXPAND | wx.LEFT, 5)
        
        resultSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, 'Results'), orient=wx.VERTICAL)
        resultSizerGrid = wx.GridBagSizer()
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Targets:'), (0, 0), (1, 1), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        resultSizerGrid.Add(self.__outputTargetList, (1, 0), (3, 1), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Results:'), (0, 1), (1, 4), wx.ALIGN_BOTTOM | wx.LEFT, 5)
        resultSizerGrid.Add(self.__outputTargetText, (1, 1), (3, 4), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.Add(wx.StaticText(self, -1, u'Console:'), (4, 0), (1, 5), wx.LEFT | wx.ALIGN_BOTTOM, 5)
        resultSizerGrid.Add(self.__outputLogText, (5, 0), (1, 5), wx.EXPAND | wx.ALL, 5)
        resultSizerGrid.AddGrowableCol(0)
        resultSizerGrid.AddGrowableCol(1)
        resultSizerGrid.AddGrowableCol(2)
        resultSizerGrid.AddGrowableCol(3)
        resultSizerGrid.AddGrowableCol(4)
        resultSizerGrid.AddGrowableRow(1)
        resultSizerGrid.AddGrowableRow(2)
        resultSizerGrid.AddGrowableRow(3)
        resultSizerGrid.AddGrowableRow(5)
        
        resultSizer.Add(resultSizerGrid, 1, wx.EXPAND)
        rootSizer = wx.GridBagSizer()
        rootSizer.Add(settingsSizer, (0, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(resultSizer, (1, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(self.__statusText, (2, 0), (1, 1), wx.ALIGN_CENTER |wx.ALL, 5)
        rootSizer.AddGrowableRow(1)
        rootSizer.AddGrowableCol(0)
        self.SetSizer(rootSizer)
    
    ## EVENT HANDLING METHODS ########################################## ##
    ## ################################################################# ##
    
    def __onTargetSelect(self, event):
        tgt = self.__outputTargetList.GetString(self.__outputTargetList.GetSelection())
        if self.__targets.has_key(tgt):
            self.__outputTargetText.SetValue(self.__targets[tgt])
        else:
            self.__outputTargetText.Clear()
    
    def __onImport(self, event):
        iWindow = ImportDialog(self, self.__coordinator, self.callback_import)
        iWindow.ShowModal()
        iWindow.Destroy()
    
    def __onToggle(self, event):
        
        if self.__modeToggle.GetValue() and self.__mode == 0:
            # switching to genemode
            self.__mode = 1
            self.__extensionText.Enable(False)
            self.__generatorText.Enable(True)
            self.__modeToggle.SetLabel(u'Switch to List Mode')
        
        elif not self.__modeToggle.GetValue() and self.__mode == 1:
            # switching to list mode
            self.__mode = 0
            self.__extensionText.Enable(True)
            self.__generatorText.Enable(False)
            self.__modeToggle.SetLabel(u'Switch to Gen Mode')
        
        else:
            #should not happen
            pass
    
    def __onStopButton(self, event):
        self.__end_scan()
    
    def __onOKButton(self, event):
        self.__start_scan()
    
    def __onTimer(self, event):
        with self.__switch_lock:
            if self.__switch[0]:
                self.__print_results(False)
            else:
                self.__timer.Stop()
                self.__print_results(True)
                self.__okButton.Enable(True)
                self.__outputTargetList.Enable(True)
                self.__coordinator.notify_stop(self.__tool, self.__displayedResults)
    
    ## DISPLAY RELATED METHODS ######################################### ##
    ## ################################################################# ##
    
    def __print_start(self):
        self.__outputTargetList.Clear()
        self.__outputTargetText.Clear()
        self.__outputLogText.Clear()
        self.__statusText.SetLabel(TEXT_RUNNING)
        #self.Layout()
    
    def __print_invalid(self):
        self.__outputTargetList.Clear()
        self.__outputTargetText.Clear()
        self.__outputLogText.Clear()
        self.__statusText.SetLabel(TEXT_INVALID)
        #self.Layout()
    
    def __print_results(self, finished):
        
        
        tmpRes = []
        
        with self.__results_lock:
            while len(self.__results) > 0:
                rs = self.__results.pop(0)
                tmpRes.append(rs)
                self.__displayedResults.append(rs)
                
        for result in tmpRes:
                
            if result.has_key(u'ERROR'):
                if result.has_key(u'HOST') and result.has_key(u'PORT'):
                    tgt = result[u'HOST'] + u':' + result[u'PORT']
                    if self.__targets.has_key(tgt):
                        self.__targets[tgt] += u'\n[ERR]\t' + result[u'ERROR'] + u'\n'
                        if self.__currentTarget == tgt:
                            self.__outputTargetText.AppendText(u'\n[ERR]\t' + result[u'ERROR'] + u'\n')
                            self.__outputTargetText.ShowPosition(self.__outputTargetText.GetLastPosition())
                    else:
                        self.__outputLogText.AppendText( u'ERROR\t' + result[u'ERROR'] + u'\n')
                else:
                    self.__outputLogText.AppendText(u'ERROR\t' + result[u'ERROR'] + u'\n')
            
            elif result.has_key(u'TARGET'):
                tgt = result[u'HOST'] + u':' + result[u'PORT']
                # add new target #
                self.__targets[tgt] = u''
                self.__currentTarget = tgt
                self.__outputTargetText.Clear()
                self.__outputLogText.AppendText( u'TARGET\tScanning ' + result[u'TARGET'] + u'\n')
                self.__outputTargetList.AppendAndEnsureVisible(tgt)
            
            elif result.has_key(u'INFO'):
                tgt = result[u'HOST'] + u':' + result[u'PORT']
                if self.__targets.has_key(tgt):
                    self.__targets[tgt] += u'\n[INF]\t' + result[u'INFO'] + u'\n'
                    if self.__currentTarget == tgt:
                        self.__outputTargetText.AppendText(u'\n[INF]\t' + result[u'INFO'] + u'\n')
                        self.__outputTargetText.ShowPosition(self.__outputTargetText.GetLastPosition())
                else:
                    self.__outputLogText.AppendText(u'INFO\t' + result[u'INFO'] + u'\n')
            
            else:
                tgt = result[u'HOST'] + u':' + result[u'PORT']
                text = u'\n[' + result[u'CODE'] + u']\t' + result[u'PATH'] + u'\n'
                if self.__targets.has_key(tgt):
                    self.__targets[tgt] += text
                    if self.__currentTarget == tgt:
                        self.__outputTargetText.AppendText(text)
                        self.__outputTargetText.ShowPosition(self.__outputTargetText.GetLastPosition())
        
        if finished:
            self.__statusText.SetLabel(TEXT_IDLE)
            #self.Layout()
    
    ## MISC METHODS #################################################### ##
    ## ################################################################# ##

    def __start_scan(self):
        with self.__switch_lock:
            if not self.__switch[0]:
                self.__load_entries()
                if self.__check_entries():                    
                    self.__switch[0] = True
                    self.__coordinator.notify_start(self.__tool)
                    self.__outputTargetList.Enable(False)
                    self.__okButton.Enable(False)
                    self.__results = []
                    self.__targets = {}
                    self.__currentTarget = u''
                    self.__displayedResults = []
                    self.__print_start()
                    
                    if self.__mode == 0:
                        ft = Fuzzer_Thread(self.__hosts, self.__openPorts, self.__roots, self.__extension, None, self.__mode, self.__results, self.__results_lock, self.__switch, self.__switch_lock, self.callback_end)
                        ft.start()
                    else:
                        ft = Fuzzer_Thread(self.__hosts, self.__openPorts, self.__roots, None, self.__generator, self.__mode, self.__results, self.__results_lock, self.__switch, self.__switch_lock, self.callback_end)
                        ft.start()
                    self.__timer.Start(REFRESH_RATE)
                else:
                    self.__print_invalid()
            else:
                pass
    
    def __end_scan(self):
        with self.__switch_lock:
            self.__switch[0] = False
    
    def __load_entries(self):
        
        self.__hosts = self.__hostsText.GetValue().strip()
        self.__openPorts = self.__portsText.GetValue().strip()
        self.__roots = self.__rootsText.GetValue().strip()
        
        if self.__mode == 0:
            self.__extension = self.__extensionText.GetValue().strip()
        else:
            self.__generator = self.__generatorText.GetValue()
    
    def __check_entries(self):
        if self.__mode == 0:
            return core_utilities.check_host_list(self.__hosts)\
            and core_utilities.check_port_list(self.__openPorts)\
            and core_utilities.check_root_list(self.__roots)\
            and core_utilities.check_ascii_string(self.__extension)
        else:
            return core_utilities.check_host_list(self.__hosts)\
            and core_utilities.check_port_list(self.__openPorts)\
            and core_utilities.check_root_list(self.__roots)\
            and core_utilities.check_generic_string(self.__generator)

    def enable(self, state):
        self.__importButton.Enable(state)

    def callback_end(self):
        self.__end_scan()
    
    def callback_import(self, hosts, ports, roots):
        if hosts != u'':
            self.__hostsText.SetValue(hosts)
        if ports != u'':
            self.__portsText.SetValue(ports)
        if roots != u'':
            self.__rootsText.SetValue(roots)


## ################################################################# ##
## CLASS: Fuzzer_Thread
## ################################################################# ##
## This class implements the thread that is in charge of running
## the fuzzer module
## ################################################################# ##
class Fuzzer_Thread(threading.Thread):
    
    def __init__(self, hosts, ports, roots, extension, generator, mode, results, results_lock, switch, switch_lock, cb_end):
        threading.Thread.__init__(self)
        self.__hosts = hosts
        self.__openPorts = ports
        self.__roots = roots
        self.__extension = extension
        self.__generator = generator
        self.__mode = mode
        self.__results = results
        self.__results_lock = results_lock
        self.__switch = switch
        self.__switch_lock = switch_lock
        self.__cb_end = cb_end
    
    def run(self):
        hosts = [h.strip() for h in self.__hosts.split(u',')]
        ports = [int(p.strip()) for p in self.__openPorts.split(u',')]
        roots = [r.strip() for r in self.__roots.split(u',')]
        if self.__mode == 0:
            modecpl = self.__extension
        else:
            modecpl = self.__generator
        module_fuzz.perform(hosts, ports, roots, self.__mode, modecpl, False, self.__results, self.__results_lock, self.__switch, self.__switch_lock)
        self.__cb_end()

## ################################################################# ##
## CLASS: ConfigDialog
## ################################################################# ##
## This class displays the configuration editor dialog
## ################################################################# ##
class ConfigDialog(wx.Dialog):
    
    def __init__(self, parent, id, title):
        wx.Dialog.__init__(self, parent, id, title, size=(600, 600), style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        cfgPanel = Config_Panel(self)
        rootSizer = wx.BoxSizer()
        rootSizer.Add(cfgPanel, 1, wx.EXPAND)
        self.SetSizer(rootSizer)


## ################################################################# ##
## CLASS: Config_Panel
## ################################################################# ##
## This class provides a GUI for editing config file
## ################################################################# ##
class Config_Panel(wx.ScrolledWindow):
    
    def __init__(self, parent):
        
        wx.ScrolledWindow.__init__(self, parent)
        self.__parent = parent
        self.SetScrollbars(20, 20, 55, 40)
        
        # Widgets ##############################################
        # HTTP Settings #
        self.__socket_timeout_spin = wx.SpinCtrl(self, -1)
        self.__socket_timeout_spin.SetRange(1, 30)
        self.__ssl_check = wx.CheckBox(self, -1, u'Enable SSL')
        self.__user_agent_text = wx.TextCtrl(self, -1)
        self.__auth_check = wx.CheckBox(self, -1, u'Enable HTTP Authentication')
        self.__auth_username_text = wx.TextCtrl(self, -1)
        self.__auth_password_text = wx.TextCtrl(self, -1)
        self.__proxy_check = wx.CheckBox(self, -1, u'Enable HTTP Proxy')
        self.__proxy_host_text = wx.TextCtrl(self, -1)
        self.__proxy_port_text = wx.TextCtrl(self, -1)
        self.__proxy_auth_check = wx.CheckBox(self, -1, u'Enable HTTP Proxy Authentication')
        self.__proxy_username_text = wx.TextCtrl(self, -1)
        self.__proxy_password_text = wx.TextCtrl(self, -1)
        self.__ids_check = wx.CheckBox(self, -1, u'Enable IDS Evasion')
        self.__ids_rp_check = wx.CheckBox(self, -1, u'Random Proxy')
        self.__ids_rp_list_text = wx.TextCtrl(self, -1)
        self.__ids_rp_list_button = wx.Button(self, -1, u'...')
        self.__ids_pause_check = wx.CheckBox(self, -1, u'Pause')
        self.__ids_pause_time_spin = wx.SpinCtrl(self, -1)
        self.__ids_pause_time_spin.SetRange(1, 120)
        # File Settings #
        self.__fuzzer_dir_list_text = wx.TextCtrl(self, -1)
        self.__fuzzer_file_list_text = wx.TextCtrl(self, -1)
        self.__custom_db_dir_text = wx.TextCtrl(self, -1)
        self.__nikto_db_dir_text = wx.TextCtrl(self, -1)
        self.__fuzzer_dir_list_button = wx.Button(self, -1, u'...')
        self.__fuzzer_file_list_button = wx.Button(self, -1, u'...')
        self.__custom_db_dir_button = wx.Button(self, -1, u'...')
        self.__nikto_db_dir_button = wx.Button(self, -1, u'...')
        # Portscan Module Settings #
        self.__nmap_check = wx.CheckBox(self, -1, u'Nmap installed')
        self.__nmap_location_text = wx.TextCtrl(self, -1)
        self.__nmap_location_button = wx.Button(self, -1, u'...')
        # Info module Settings #
        self.__live_id_text = wx.TextCtrl(self, -1)
        # Urlscan Module Settings #
        self.__scan_threads_spin = wx.SpinCtrl(self, -1)
        self.__scan_threads_spin.SetRange(1, 50)
        self.__scan_show_codes_text = wx.TextCtrl(self, -1)
        self.__use_db_nikto_check = wx.CheckBox(self, -1, u'Nikto Database')
        self.__use_db_custom_check = wx.CheckBox(self, -1, u'SCRT Database')
        # Spider Module Settings #
        self.__spider_threads_spin = wx.SpinCtrl(self, -1)
        self.__spider_threads_spin.SetRange(1, 50)
        self.__use_robots_check = wx.CheckBox(self, -1, u'Allow spider to (mis)use robots.txt')
        # Fuzer Module Settings #
        self.__fuzz_threads_spin = wx.SpinCtrl(self, -1)
        self.__fuzz_threads_spin.SetRange(1, 50)
        self.__fuzz_show_codes_text = wx.TextCtrl(self, -1)
        self.__fuzz_method_text = wx.TextCtrl(self, -1)
        # Buttons #
        self.__applyButton = wx.Button(self, wx.ID_APPLY, u'&Apply')
        self.__refreshButton = wx.Button(self, wx.ID_REFRESH, u'Refresh')
        self.__closeButton = wx.Button(self, wx.ID_CLOSE, u'&Close')
        self.__command_text = wx.StaticText(self, -1, u'')
        
        # Event Bindings #######################################
        self.Bind(wx.EVT_BUTTON, self.__onRefreshButton, id=self.__refreshButton.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onApplyButton, id=self.__applyButton.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onCloseButton, id=self.__closeButton.GetId())
        self.Bind(wx.EVT_CHECKBOX, self.__onAuthCheck, id=self.__auth_check.GetId())
        self.Bind(wx.EVT_CHECKBOX, self.__onProxyCheck, id=self.__proxy_check.GetId())
        self.Bind(wx.EVT_CHECKBOX, self.__onProxyAuthCheck, id=self.__proxy_auth_check.GetId())
        self.Bind(wx.EVT_CHECKBOX, self.__onIdsCheck, id=self.__ids_check.GetId())
        self.Bind(wx.EVT_CHECKBOX, self.__onIdsRpCheck, id=self.__ids_rp_check.GetId())
        self.Bind(wx.EVT_CHECKBOX, self.__onIdsPauseCheck, id=self.__ids_pause_check.GetId())
        self.Bind(wx.EVT_CHECKBOX, self.__onNmapCheck, id=self.__nmap_check.GetId())
        self.Bind(wx.EVT_CHECKBOX, self.__onNiktoCheck, id=self.__use_db_nikto_check.GetId())
        self.Bind(wx.EVT_CHECKBOX, self.__onCustomCheck, id=self.__use_db_custom_check.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onFuzzerDirListButton, id=self.__fuzzer_dir_list_button.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onFuzzerFileListButton, id=self.__fuzzer_file_list_button.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onCustomDbDirButton, id=self.__custom_db_dir_button.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onNiktoDbDirButton, id=self.__nikto_db_dir_button.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onNmapLocationButton, id=self.__nmap_location_button.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onIdsRpListButton, id=self.__ids_rp_list_button.GetId())
        
        # Layout ###############################################
        httpGridSizer = wx.GridBagSizer()
        httpGridSizer.AddGrowableCol(0)
        
        httpGridSizer.Add(self.__ssl_check, (0, 0), (1, 2),  wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(wx.StaticText(self, -1, u'User-Agent:'), (1, 0), (1, 1), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(wx.StaticText(self, -1, u'Socket Timeout:'), (1, 1), (1, 1), wx.EXPAND | wx.RIGHT, 10)
        httpGridSizer.Add(self.__user_agent_text, (2, 0), (1, 1),  wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(self.__socket_timeout_spin, (2, 1), (1, 1),  wx.EXPAND | wx.RIGHT, 10)
        
        httpGridSizer.Add(self.__auth_check, (3, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(wx.StaticText(self, -1, u'Username:'), (4, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(self.__auth_username_text, (5, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(wx.StaticText(self, -1, u'Password:'), (6, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(self.__auth_password_text, (7, 0), (1, 2), wx.EXPAND | wx.EXPAND | wx.LEFT | wx.RIGHT, 10)

        httpGridSizer.Add(self.__proxy_check, (8, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(wx.StaticText(self, -1, u'Host:'), (9, 0), (1, 1), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(wx.StaticText(self, -1, u'Port:'), (9, 1), (1, 1), wx.EXPAND | wx.RIGHT, 10)
        httpGridSizer.Add(self.__proxy_host_text, (10, 0), (1, 1), wx.EXPAND | wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(self.__proxy_port_text, (10, 1), (1, 1), wx.EXPAND | wx.EXPAND | wx.RIGHT, 10)

        httpGridSizer.Add(self.__proxy_auth_check, (11, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(wx.StaticText(self, -1, u'Username:'), (12, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(self.__proxy_username_text, (13, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(wx.StaticText(self, -1, u'Password:'), (14, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(self.__proxy_password_text, (15, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)

        httpGridSizer.Add(self.__ids_check, (16, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(self.__ids_rp_check, (17, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(wx.StaticText(self, -1, u'Proxy List:'), (18, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(self.__ids_rp_list_text, (19, 0), (1, 1), wx.EXPAND | wx.LEFT, 10)
        httpGridSizer.Add(self.__ids_rp_list_button, (19, 1), (1, 1), wx.EXPAND | wx.RIGHT, 10)
        httpGridSizer.Add(self.__ids_pause_check, (20, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(wx.StaticText(self, -1, u'Maximum Pause Time [seconds]:'), (21, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        httpGridSizer.Add(self.__ids_pause_time_spin, (22, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        
        fileGridSizer = wx.GridBagSizer()
        fileGridSizer.AddGrowableCol(0)
        
        fileGridSizer.Add(wx.StaticText(self, -1, u'Fuzzer Dirs List:'), (0, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        fileGridSizer.Add(self.__fuzzer_dir_list_text, (1, 0), (1, 1), wx.EXPAND | wx.LEFT, 10)
        fileGridSizer.Add(self.__fuzzer_dir_list_button, (1, 1), (1, 1), wx.EXPAND | wx.RIGHT, 10)
        fileGridSizer.Add(wx.StaticText(self, -1, u'Fuzzer File List:'), (2, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        fileGridSizer.Add(self.__fuzzer_file_list_text, (3, 0), (1, 1), wx.EXPAND | wx.LEFT, 10)
        fileGridSizer.Add(self.__fuzzer_file_list_button, (3, 1), (1, 1), wx.EXPAND | wx.RIGHT, 10)
        fileGridSizer.Add(self.__use_db_nikto_check, (4, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        fileGridSizer.Add(self.__nikto_db_dir_text, (5, 0), (1, 1), wx.EXPAND | wx.LEFT, 10)
        fileGridSizer.Add(self.__nikto_db_dir_button, (5, 1), (1, 1), wx.EXPAND | wx.RIGHT, 10)
        fileGridSizer.Add(self.__use_db_custom_check, (6, 0), (1, 2), wx.EXPAND | wx.LEFT, 10)
        fileGridSizer.Add(self.__custom_db_dir_text, (7, 0), (1, 1), wx.EXPAND | wx.LEFT, 10)
        fileGridSizer.Add(self.__custom_db_dir_button, (7, 1), (1, 1), wx.EXPAND | wx.RIGHT, 10)
        
        codesGridSizer = wx.GridBagSizer()
        codesGridSizer.AddGrowableCol(0)
        
        codesGridSizer.Add(wx.StaticText(self, -1, u'URL Scanner Show Codes:'), (0, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        codesGridSizer.Add(self.__scan_show_codes_text, (1, 0), (1, 2), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        codesGridSizer.Add(wx.StaticText(self, -1, u'Fuzzer Show Codes:'), (2, 0), (1, 1), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        codesGridSizer.Add(wx.StaticText(self, -1, u'Fuzzer Method:'), (2, 1), (1, 1), wx.EXPAND | wx.RIGHT, 10)
        codesGridSizer.Add(self.__fuzz_show_codes_text, (3, 0), (1, 1), wx.EXPAND | wx.LEFT | wx.RIGHT, 10)
        codesGridSizer.Add(self.__fuzz_method_text, (3, 1), (1, 1), wx.EXPAND | wx.RIGHT, 10)
        
        threadsGridSizer = wx.GridBagSizer()
        threadsGridSizer.AddGrowableCol(0)
        threadsGridSizer.AddGrowableCol(1)
        threadsGridSizer.AddGrowableCol(2)
        
        threadsGridSizer.Add(wx.StaticText(self, -1, u'Scanner Threads:'), (0, 0), (1, 1), wx.EXPAND | wx.RIGHT | wx.LEFT, 10)
        threadsGridSizer.Add(wx.StaticText(self, -1, u'Fuzzer Threads:'), (0, 1), (1, 1), wx.EXPAND | wx.RIGHT | wx.LEFT, 10)
        threadsGridSizer.Add(wx.StaticText(self, -1, u'Spider Threads:'), (0, 2), (1, 1), wx.EXPAND | wx.RIGHT | wx.LEFT, 10)
        threadsGridSizer.Add(self.__scan_threads_spin, (1, 0), (1, 1), wx.EXPAND | wx.RIGHT | wx.LEFT, 10)
        threadsGridSizer.Add(self.__fuzz_threads_spin, (1, 1), (1, 1), wx.EXPAND | wx.RIGHT | wx.LEFT, 10)
        threadsGridSizer.Add(self.__spider_threads_spin, (1, 2), (1, 1), wx.EXPAND | wx.RIGHT | wx.LEFT, 10)
        
        spiderGridSizer = wx.GridBagSizer()
        spiderGridSizer.AddGrowableCol(0)
        
        spiderGridSizer.Add(self.__use_robots_check, (0, 0), (1, 1), wx.EXPAND | wx.RIGHT | wx.LEFT, 10)
        
        portscanGridSizer = wx.GridBagSizer()
        portscanGridSizer.AddGrowableCol(0)
        
        portscanGridSizer.Add(self.__nmap_check, (0, 0), (1, 2),  wx.EXPAND | wx.RIGHT | wx.LEFT, 10)
        portscanGridSizer.Add(self.__nmap_location_text, (1, 0), (1, 1),  wx.EXPAND | wx.LEFT, 10)
        portscanGridSizer.Add(self.__nmap_location_button, (1, 1), (1, 1),  wx.EXPAND | wx.RIGHT, 10)
        
        infoGridSizer = wx.GridBagSizer()
        infoGridSizer.AddGrowableCol(0)
        
        infoGridSizer.Add(wx.StaticText(self, -1, u'Live Search ID:'), (0, 0), (1, 1), wx.EXPAND | wx.RIGHT | wx.LEFT, 10)
        infoGridSizer.Add(self.__live_id_text, (1, 0), (1, 1), wx.EXPAND | wx.RIGHT | wx.LEFT, 10)
        
        commandsSizer = wx.BoxSizer()
        commandsSizerGrid = wx.GridBagSizer()
        commandsSizerGrid.AddGrowableCol(1)
        commandsSizerGrid.Add(self.__command_text, (0, 0), (1, 1), wx.EXPAND | wx.ALIGN_LEFT | wx.ALL, 5)
        commandsSizerGrid.Add(wx.Panel(self, -1), (0, 1), (1, 1), wx.EXPAND | wx.ALL, 5)
        commandsSizerGrid.Add(self.__refreshButton, (0, 2), (1, 1), wx.ALIGN_RIGHT | wx.ALL, 5)
        commandsSizerGrid.Add(self.__closeButton, (0, 3), (1, 1), wx.ALIGN_RIGHT | wx.ALL, 5)
        commandsSizerGrid.Add(self.__applyButton, (0, 4), (1, 1), wx.ALIGN_RIGHT | wx.ALL, 5)
        commandsSizer.Add(commandsSizerGrid, 1, wx.EXPAND)
        
        rootSizer = wx.GridBagSizer()
        rootSizer.Add(wx.StaticText(self, -1, u'Configuration settings. Do not forget to confirm changes using \'Apply\' button!'), (0, 0), (1, 1), wx.EXPAND | wx.TOP | wx.LEFT | wx.RIGHT, 5)
        rootSizer.Add(wx.StaticLine(self, -1), (1, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(httpGridSizer, (2, 0), (1, 1), wx.EXPAND)
        rootSizer.Add(wx.StaticLine(self, -1), (3, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(fileGridSizer, (4, 0), (1, 1), wx.EXPAND)
        rootSizer.Add(wx.StaticLine(self, -1), (5, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(codesGridSizer, (6, 0), (1, 1), wx.EXPAND)
        rootSizer.Add(wx.StaticLine(self, -1), (7, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(threadsGridSizer, (8, 0), (1, 1), wx.EXPAND)
        rootSizer.Add(wx.StaticLine(self, -1), (9, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(spiderGridSizer, (10, 0), (1, 1), wx.EXPAND)
        rootSizer.Add(wx.StaticLine(self, -1), (11, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(portscanGridSizer, (12, 0), (1, 1), wx.EXPAND)
        rootSizer.Add(wx.StaticLine(self, -1), (13, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        rootSizer.Add(infoGridSizer, (14, 0), (1, 1), wx.EXPAND)
        rootSizer.Add(commandsSizer, (15, 0), (1, 1), wx.EXPAND | wx.ALL, 10)
        rootSizer.AddGrowableCol(0)
        self.SetSizer(rootSizer)
        
        self.__load_config()
    
    def __onRefreshButton(self, event):
        self.__load_config()

    def __onApplyButton(self, event):
        self.__save_config()
        self.__parent.Close()

    def __onCloseButton(self, event):
        self.__parent.Close()

    def __onProxyCheck(self, event):
        if self.__proxy_check.GetValue():
            self.__proxy_host_text.Enable(True)
            self.__proxy_port_text.Enable(True)
        else:
            self.__proxy_host_text.Enable(False)
            self.__proxy_port_text.Enable(False)
    
    def __onProxyAuthCheck(self, event):
        if self.__proxy_auth_check.GetValue():
            self.__proxy_username_text.Enable(True)
            self.__proxy_password_text.Enable(True)
        else:
            self.__proxy_username_text.Enable(False)
            self.__proxy_password_text.Enable(False)
    
    def __onAuthCheck(self, event):
        if self.__auth_check.GetValue():
            self.__auth_username_text.Enable(True)
            self.__auth_password_text.Enable(True)
        else:
            self.__auth_username_text.Enable(False)
            self.__auth_password_text.Enable(False)
    
    def __onIdsCheck(self, event):
        if self.__ids_check.GetValue():
            self.__ids_rp_check.Enable(True)
            if self.__ids_rp_check.GetValue():
                self.__ids_rp_list_text.Enable(True)
                self.__ids_rp_list_button.Enable(True)
            else:
                self.__ids_rp_list_text.Enable(False)
            self.__ids_pause_check.Enable(True)
            if self.__ids_pause_check.GetValue():
                self.__ids_pause_time_spin.Enable(True)
            else:
                self.__ids_pause_time_spin.Enable(False)
        else:
            self.__ids_rp_check.Enable(False)
            self.__ids_rp_list_text.Enable(False)
            self.__ids_rp_list_button.Enable(False)
            self.__ids_pause_check.Enable(False)
            self.__ids_pause_time_spin.Enable(False)
    
    def __onIdsRpCheck(self, event):
        if self.__ids_rp_check.GetValue():
            self.__ids_rp_list_text.Enable(True)
            self.__ids_rp_list_button.Enable(True)
        else:
            self.__ids_rp_list_text.Enable(False)
            self.__ids_rp_list_button.Enable(False)
    
    def __onIdsPauseCheck(self, event):
        if self.__ids_pause_check.GetValue():
            self.__ids_pause_time_spin.Enable(True)
        else:
            self.__ids_pause_time_spin.Enable(False)
    
    def __onNiktoCheck(self, event):
        if self.__use_db_nikto_check.GetValue():
            self.__nikto_db_dir_text.Enable(True)
            self.__nikto_db_dir_button.Enable(True)
        else:
            self.__nikto_db_dir_text.Enable(False)
            self.__nikto_db_dir_button.Enable(False)
            
    def __onCustomCheck(self, event):
        if self.__use_db_custom_check.GetValue():
            self.__custom_db_dir_text.Enable(True)
            self.__custom_db_dir_button.Enable(True)
        else:
            self.__custom_db_dir_text.Enable(False)
            self.__custom_db_dir_button.Enable(False)
    
    def __onNmapCheck(self, event):
        if self.__nmap_check.GetValue():
            self.__nmap_location_text.Enable(True)
            self.__nmap_location_button.Enable(True)
        else:
            self.__nmap_location_text.Enable(False)
            self.__nmap_location_button.Enable(False)
    
    def __onFuzzerDirListButton(self, event):
        dlg = wx.FileDialog(self, "Choose File", style=wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.__fuzzer_dir_list_text.SetValue(dlg.GetPath())
        dlg.Destroy()

    def __onFuzzerFileListButton(self, event):
        dlg = wx.FileDialog(self, "Choose File", style=wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.__fuzzer_file_list_text.SetValue(dlg.GetPath())
        dlg.Destroy()
        
    def __onCustomDbDirButton(self, event):
        dlg = wx.DirDialog(self, "Choose Directory", style=wx.DD_DIR_MUST_EXIST)
        if dlg.ShowModal() == wx.ID_OK:
            self.__custom_db_dir_text.SetValue(dlg.GetPath())
        dlg.Destroy()
    
    def __onNiktoDbDirButton(self, event):
        dlg = wx.DirDialog(self, "Choose Directory", style=wx.DD_DIR_MUST_EXIST)
        if dlg.ShowModal() == wx.ID_OK:
            self.__nikto_db_dir_text.SetValue(dlg.GetPath())
        dlg.Destroy()

    def __onNmapLocationButton(self, event):
        dlg = wx.FileDialog(self, "Choose File", style=wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.__nmap_location_text.SetValue(dlg.GetPath())
        dlg.Destroy()

    def __onIdsRpListButton(self, event):
        dlg = wx.FileDialog(self, "Choose File", style=wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.__ids_rp_list_text.SetValue(dlg.GetPath())
        dlg.Destroy()
    __onIdsRpListButton

    def __load_config(self):
        
        configParser = core_file.cfg_start_get()
        self.__fuzzer_dir_list_text.SetValue(core_file.cfg_get_fuzzer_dir_list(configParser))
        self.__fuzzer_file_list_text.SetValue(core_file.cfg_get_fuzzer_file_list(configParser))
        self.__nikto_db_dir_text.SetValue(core_file.cfg_get_nikto_db_dir(configParser))
        self.__custom_db_dir_text.SetValue(core_file.cfg_get_custom_db_dir(configParser))
        self.__proxy_check.SetValue(core_file.cfg_get_proxy(configParser))
        self.__proxy_auth_check.SetValue(core_file.cfg_get_proxy_auth(configParser))
        self.__proxy_host_text.SetValue(core_file.cfg_get_proxy_host(configParser))
        self.__proxy_port_text.SetValue(unicode(core_file.cfg_get_proxy_port(configParser)))
        self.__proxy_username_text.SetValue(core_file.cfg_get_proxy_username(configParser))
        self.__proxy_password_text.SetValue(core_file.cfg_get_proxy_password(configParser))
        self.__socket_timeout_spin.SetValue(core_file.cfg_get_socket_timeout(configParser))
        self.__ids_check.SetValue(core_file.cfg_get_ids(configParser))
        self.__ids_rp_check.SetValue(core_file.cfg_get_ids_rp(configParser))
        self.__ids_rp_list_text.SetValue(core_file.cfg_get_ids_rp_list(configParser))
        self.__ids_pause_check.SetValue(core_file.cfg_get_ids_pause(configParser))
        self.__ids_pause_time_spin.SetValue(core_file.cfg_get_ids_pause_time(configParser))
        self.__auth_check.SetValue(core_file.cfg_get_auth(configParser))
        self.__ssl_check.SetValue(core_file.cfg_get_ssl(configParser))
        self.__auth_username_text.SetValue(core_file.cfg_get_auth_username(configParser))
        self.__auth_password_text.SetValue(core_file.cfg_get_auth_password(configParser))
        self.__user_agent_text.SetValue(core_file.cfg_get_user_agent(configParser))
        self.__nmap_check.SetValue(core_file.cfg_get_nmap(configParser))
        self.__nmap_location_text.SetValue(core_file.cfg_get_nmap_location(configParser))
        self.__live_id_text.SetValue(core_file.cfg_get_live_id(configParser))
        self.__use_db_nikto_check.SetValue(core_file.cfg_get_use_db_nikto(configParser))
        self.__use_db_custom_check.SetValue(core_file.cfg_get_use_db_custom(configParser))
        self.__scan_show_codes_text.SetValue(core_file.cfg_get_scan_show_codes(configParser))
        self.__scan_threads_spin.SetValue(core_file.cfg_get_scan_threads(configParser))
        self.__fuzz_show_codes_text.SetValue(core_file.cfg_get_fuzz_show_codes(configParser))
        self.__fuzz_threads_spin.SetValue(core_file.cfg_get_fuzz_threads(configParser))
        self.__spider_threads_spin.SetValue(core_file.cfg_get_spider_threads(configParser))
        self.__fuzz_method_text.SetValue(core_file.cfg_get_fuzz_method(configParser))
        self.__use_robots_check.SetValue(core_file.cfg_get_use_robots(configParser))
        core_file.cfg_end_get(configParser)
        
        if self.__proxy_check.GetValue():
            self.__proxy_host_text.Enable(True)
            self.__proxy_port_text.Enable(True)
        else:
            self.__proxy_host_text.Enable(False)
            self.__proxy_port_text.Enable(False)
    
        if self.__proxy_auth_check.GetValue():
            self.__proxy_username_text.Enable(True)
            self.__proxy_password_text.Enable(True)
        else:
            self.__proxy_username_text.Enable(False)
            self.__proxy_password_text.Enable(False)

        if self.__auth_check.GetValue():
            self.__auth_username_text.Enable(True)
            self.__auth_password_text.Enable(True)
        else:
            self.__auth_username_text.Enable(False)
            self.__auth_password_text.Enable(False)
    
        if self.__ids_check.GetValue():
            self.__ids_rp_check.Enable(True)
            self.__ids_rp_list_text.Enable(True)
            self.__ids_rp_list_button.Enable(True)
            self.__ids_pause_check.Enable(True)
            self.__ids_pause_time_spin.Enable(True)
        else:
            self.__ids_rp_check.Enable(False)
            self.__ids_rp_list_text.Enable(False)
            self.__ids_rp_list_button.Enable(False)
            self.__ids_pause_check.Enable(False)
            self.__ids_pause_time_spin.Enable(False)

        if self.__nmap_check.GetValue():
            self.__nmap_location_text.Enable(True)
            self.__nmap_location_button.Enable(True)
        else:
            self.__nmap_location_text.Enable(False)
            self.__nmap_location_button.Enable(False)

        if self.__use_db_nikto_check.GetValue():
            self.__nikto_db_dir_text.Enable(True)
            self.__nikto_db_dir_button.Enable(True)
        else:
            self.__nikto_db_dir_text.Enable(False)
            self.__nikto_db_dir_button.Enable(False)
        
        if self.__use_db_custom_check.GetValue():
            self.__custom_db_dir_text.Enable(True)
            self.__custom_db_dir_button.Enable(True)
        else:
            self.__custom_db_dir_text.Enable(False)
            self.__custom_db_dir_button.Enable(False)


    def __save_config(self):
        
        configParser = core_file.cfg_start_set()
        
        core_file.cfg_set_fuzzer_dir_list(configParser, self.__fuzzer_dir_list_text.GetValue())
        core_file.cfg_set_fuzzer_file_list(configParser, self.__fuzzer_file_list_text.GetValue())
        core_file.cfg_set_nikto_db_dir(configParser, self.__nikto_db_dir_text.GetValue())
        core_file.cfg_set_custom_db_dir(configParser, self.__custom_db_dir_text.GetValue())
        core_file.cfg_set_proxy(configParser, self.__proxy_check.GetValue())
        core_file.cfg_set_proxy_auth(configParser, self.__proxy_auth_check.GetValue())
        core_file.cfg_set_proxy_host(configParser, self.__proxy_host_text.GetValue())
        
        if self.__proxy_port_text.GetValue() != '' and core_utilities.check_port_string(self.__proxy_port_text.GetValue()):
            core_file.cfg_set_proxy_port(configParser, self.__proxy_port_text.GetValue())
        else:
            core_file.cfg_set_proxy_port(configParser, u'0')
        
        core_file.cfg_set_proxy_username(configParser, self.__proxy_username_text.GetValue())
        core_file.cfg_set_proxy_password(configParser, self.__proxy_password_text.GetValue())
        core_file.cfg_set_socket_timeout(configParser, self.__socket_timeout_spin.GetValue())
        core_file.cfg_set_ids(configParser, self.__ids_check.GetValue())
        core_file.cfg_set_ids_rp(configParser, self.__ids_rp_check.GetValue())
        core_file.cfg_set_ids_rp_list(configParser, self.__ids_rp_list_text.GetValue())
        core_file.cfg_set_ids_pause(configParser, self.__ids_pause_check.GetValue())
        core_file.cfg_set_ids_pause_time(configParser, self.__ids_pause_time_spin.GetValue())
        core_file.cfg_set_auth(configParser, self.__auth_check.GetValue())
        core_file.cfg_set_ssl(configParser, self.__ssl_check.GetValue())
        core_file.cfg_set_auth_username(configParser, self.__auth_username_text.GetValue())
        core_file.cfg_set_auth_password(configParser, self.__auth_password_text.GetValue())
        core_file.cfg_set_user_agent(configParser, self.__user_agent_text.GetValue())
        core_file.cfg_set_nmap(configParser, self.__nmap_check.GetValue())
        core_file.cfg_set_nmap_location(configParser, self.__nmap_location_text.GetValue())
        core_file.cfg_set_live_id(configParser, self.__live_id_text.GetValue())
        core_file.cfg_set_use_db_nikto(configParser, self.__use_db_nikto_check.GetValue())
        core_file.cfg_set_use_db_custom(configParser, self.__use_db_custom_check.GetValue())
        core_file.cfg_set_scan_show_codes(configParser, self.__scan_show_codes_text.GetValue())
        core_file.cfg_set_scan_threads(configParser, self.__scan_threads_spin.GetValue())
        core_file.cfg_set_fuzz_show_codes(configParser, self.__fuzz_show_codes_text.GetValue())
        core_file.cfg_set_fuzz_threads(configParser, self.__fuzz_threads_spin.GetValue())
        core_file.cfg_set_spider_threads(configParser, self.__spider_threads_spin.GetValue())
        core_file.cfg_set_fuzz_method(configParser, self.__fuzz_method_text.GetValue())
        core_file.cfg_set_use_robots(configParser, self.__use_robots_check.GetValue())
        core_file.cfg_end_set(configParser)


## ################################################################# ##
## CLASS: ExportDialog
## ################################################################# ##
## This class displays the report exporting dialog
## ################################################################# ##
class ExportDialog(wx.Dialog):
    
    def __init__(self, parent, results):
        wx.Dialog.__init__(self, parent, -1, u'Export Report...', size=(450, 240))
        expPanel = Export_Panel(self, results)
        rootSizer = wx.BoxSizer()
        rootSizer.Add(expPanel, 1, wx.EXPAND)
        self.SetSizer(rootSizer)

## ################################################################# ##
## CLASS: Export_Panel
## ################################################################# ##
## This class provides a GUI for exporting reports
## ################################################################# ##
class Export_Panel(wx.Panel):
    
    def __init__(self, parent, results):
        
        wx.Panel.__init__(self, parent)
        self.__parent = parent
        self.__results = results
        
        formats = EXPORT_FORMATS
        
        # Widgets ##############################################
        self.__openPortscan_check = wx.CheckBox(self, -1, u'Port Scanner')
        self.__openPortscan_check.SetValue(True)
        self.__info_check = wx.CheckBox(self, -1, u'Info')
        self.__info_check.SetValue(True)
        self.__spider_check = wx.CheckBox(self, -1, u'Spider')
        self.__spider_check.SetValue(True)
        self.__urlscan_check = wx.CheckBox(self, -1, u'URL Scanner')
        self.__urlscan_check.SetValue(True)
        self.__fuzz_check = wx.CheckBox(self, -1, u'Fuzzer')
        self.__fuzz_check.SetValue(True)
        self.__file_text = wx.TextCtrl(self, -1)
        self.__file_button = wx.Button(self, wx.ID_OPEN, u'&Open')
        self.__format_combo = wx.ComboBox(self, -1, style=wx.CB_READONLY, choices=formats)
        self.__format_combo.SetValue(formats[0])
        self.__apply_button = wx.Button(self, wx.ID_APPLY, u'&Apply')
        self.__close_button = wx.Button(self, wx.ID_CLOSE, u'&Close')
        self.__command_text = wx.StaticText(self, -1, u'')
        
        # Event Bindings ##############################################
        self.Bind(wx.EVT_BUTTON, self.__onCloseButton, id=self.__close_button.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onBrowseButton, id=self.__file_button.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onApplyButton, id=self.__apply_button.GetId())
        
        # Layout Management ##############################################
        rootSizer =  wx.BoxSizer(orient=wx.VERTICAL)
        boxesGridSizer = wx.GridBagSizer()
        boxesGridSizer.Add(wx.StaticText(self, -1, u'Export Results:'), (0, 0), (1, 5), wx.EXPAND | wx.TOP, 5)
        boxesGridSizer.Add(self.__openPortscan_check, (1, 0), (1, 1), wx.EXPAND | wx.ALL, 5)
        boxesGridSizer.Add(self.__info_check, (1, 1), (1, 1), wx.EXPAND | wx.ALL, 5)
        boxesGridSizer.Add(self.__spider_check, (1, 2), (1, 1), wx.EXPAND | wx.ALL, 5)
        boxesGridSizer.Add(self.__urlscan_check, (1, 3), (1, 1), wx.EXPAND | wx.ALL, 5)
        boxesGridSizer.Add(self.__fuzz_check, (1, 4), (1, 1), wx.EXPAND | wx.ALL, 5)
        boxesGridSizer.AddGrowableCol(0)
        boxesGridSizer.AddGrowableCol(1)
        boxesGridSizer.AddGrowableCol(2)
        boxesGridSizer.AddGrowableCol(3)
        boxesGridSizer.AddGrowableCol(4)
        fileGridSizer = wx.GridBagSizer()
        fileGridSizer.Add(wx.StaticText(self, -1, u'Output File:'), (0, 0), (1, 2), wx.EXPAND)
        fileGridSizer.Add(self.__file_text, (1, 0), (1, 1), wx.EXPAND)
        fileGridSizer.Add(self.__file_button, (1, 1), (1, 1))
        fileGridSizer.AddGrowableCol(0)
        formatGridSizer = wx.GridBagSizer()
        formatGridSizer.Add(wx.StaticText(self, -1, u'Output File Format:'), (0, 0), (1, 2), wx.EXPAND)
        formatGridSizer.Add(self.__format_combo, (1, 0), (1, 2), wx.EXPAND)
        formatGridSizer.AddGrowableCol(0)
        buttonGridSizer = wx.GridBagSizer()
        buttonGridSizer.Add(self.__command_text, (0, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL)
        buttonGridSizer.Add(self.__close_button, (0, 1), (1, 1), wx.EXPAND)
        buttonGridSizer.Add(self.__apply_button, (0, 2), (1, 1), wx.EXPAND)
        buttonGridSizer.AddGrowableCol(0)
        rootSizer.Add(boxesGridSizer, 0, wx.EXPAND | wx.TOP | wx.RIGHT | wx.LEFT, 3)
        rootSizer.Add(fileGridSizer, 0, wx.EXPAND | wx.TOP | wx.RIGHT | wx.LEFT, 3)
        rootSizer.Add(formatGridSizer, 0, wx.EXPAND | wx.TOP | wx.RIGHT | wx.LEFT, 3)
        rootSizer.Add(buttonGridSizer, 0, wx.EXPAND | wx.TOP | wx.LEFT, 10)
        self.SetSizer(rootSizer)
    
    
    def __onCloseButton(self, event):
        self.__parent.Close()
        
    def __onBrowseButton(self, event):
        dlg = wx.FileDialog(self, "Output File", style=wx.SAVE)
        if dlg.ShowModal() == wx.ID_OK:
            self.__file_text.SetValue(dlg.GetPath())
        dlg.Destroy()
        
    def __onApplyButton(self, event):
        
        file = self.__file_text.GetValue()
        if not core_utilities.check_save_file_path(file):
            self.__command_text.SetLabel(u'Invalid Filename!')
        else:
            
            if self.__results[MODULE_INFO] and self.__info_check.GetValue():
                info = self.__results[MODULE_INFO]
            else:
                info = None
            
            if self.__results[MODULE_PSCAN] and self.__openPortscan_check.GetValue():
                pscan = self.__results[MODULE_PSCAN]
            else:
                pscan = None
            
            if self.__results[MODULE_USCAN] and self.__urlscan_check.GetValue():
                uscan = self.__results[MODULE_USCAN]
            else:
                uscan = None
            
            if self.__results[MODULE_FUZZ] and self.__fuzz_check.GetValue():
                fuzz = self.__results[MODULE_FUZZ]
            else:
                fuzz = None
            
            if self.__results[MODULE_SPIDER] and self.__spider_check.GetValue():
                spider = self.__results[MODULE_SPIDER]
            else:
                spider = None
            
            success = export.exp_report(file, self.__format_combo.GetValue(), info, pscan, uscan, fuzz, spider)
            
            if success:
                self.__parent.Close()
            else:
                self.__command_text.SetLabel(u'Export Failed')


## ################################################################# ##
## CLASS: ImportDialog
## ################################################################# ##
## This class displays the import dialog used by URL scanner and
## fuzzer panels to import results from other modules
## ################################################################# ##
class ImportDialog(wx.Dialog):
    
    def __init__(self, parent, coordinator, cb_import):
        wx.Dialog.__init__(self, parent, -1, u'Import Results...', size=(600, 400), style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        importPanel = Import_Panel(self, coordinator, cb_import)
        rootSizer = wx.BoxSizer()
        rootSizer.Add(importPanel, 1, wx.EXPAND)
        self.SetSizer(rootSizer)

class Import_Panel(wx.Panel):
    
    def __init__(self, parent, coordinator, cb_import):
        
        wx.Panel.__init__(self, parent)
        
        # Variables ##############################################
        self.__parent = parent
        self.__coordinator = coordinator
        self.__callback = cb_import
        hosts = self.__coordinator.get_info_results()
        ports = self.__coordinator.get_portscan_results()
        roots = self.__coordinator.get_spider_results()
        
        # Widgets ##############################################
        self.__hosts_list = wx.ListBox(self, -1, choices=hosts, style=wx.LB_MULTIPLE)
        self.__openPorts_list = wx.ListBox(self, -1, choices=ports, style=wx.LB_MULTIPLE)
        self.__roots_list = wx.ListBox(self, -1, choices=roots, style=wx.LB_MULTIPLE)
        self.__apply_button = wx.Button(self, wx.ID_APPLY, u'&Apply')
        self.__close_button = wx.Button(self, wx.ID_CLOSE, u'&Close')
        self.__command_text = wx.StaticText(self, -1, u'')
        
        # Event Bindings ##############################################
        self.Bind(wx.EVT_BUTTON, self.__onCloseButton, id=self.__close_button.GetId())
        self.Bind(wx.EVT_BUTTON, self.__onApplyButton, id=self.__apply_button.GetId())
        listsSizerGrid = wx.GridBagSizer()
        listsSizerGrid.Add(wx.StaticText(self, -1, u'hosts:'), (0, 0), (1, 1), wx.ALIGN_BOTTOM | wx.ALIGN_LEFT | wx.LEFT, 5)
        listsSizerGrid.Add(wx.StaticText(self, -1, u'roots:'), (0, 1), (1, 1), wx.ALIGN_BOTTOM | wx.ALIGN_LEFT | wx.LEFT, 5)
        listsSizerGrid.Add(wx.StaticText(self, -1, u'ports:'), (0, 2), (1, 1), wx.ALIGN_BOTTOM | wx.ALIGN_LEFT | wx.LEFT, 5)
        listsSizerGrid.Add(self.__hosts_list, (1, 0), (1, 1), wx.EXPAND | wx.LEFT, 5)
        listsSizerGrid.Add(self.__roots_list, (1, 1), (1, 1), wx.EXPAND | wx.LEFT, 5)
        listsSizerGrid.Add(self.__openPorts_list, (1, 2), (1, 1), wx.EXPAND | wx.LEFT, 5)
        listsSizerGrid.AddGrowableCol(0)
        listsSizerGrid.AddGrowableCol(1)
        listsSizerGrid.AddGrowableRow(1)
        buttonGridSizer = wx.GridBagSizer()
        buttonGridSizer.Add(self.__command_text, (0, 0), (1, 1), wx.ALIGN_CENTER_VERTICAL)
        buttonGridSizer.Add(self.__close_button, (0, 1), (1, 1), wx.EXPAND)
        buttonGridSizer.Add(self.__apply_button, (0, 2), (1, 1), wx.EXPAND)
        buttonGridSizer.AddGrowableCol(0)
        rootSizer = wx.BoxSizer(wx.VERTICAL)
        rootSizer.Add(listsSizerGrid, 1, wx.EXPAND)
        rootSizer.Add(buttonGridSizer, 0, wx.EXPAND | wx.TOP, 5)
        self.SetSizer(rootSizer)
        
    def __onCloseButton(self, event):
        self.__parent.Close()
    
    def __onApplyButton(self, event):
        hosts_list = [self.__hosts_list.GetString(index) for index in self.__hosts_list.GetSelections()]
        ports_list = [self.__openPorts_list.GetString(index) for index in self.__openPorts_list.GetSelections()]
        roots_list = [self.__roots_list.GetString(index) for index in self.__roots_list.GetSelections()]
        hosts_string = u','.join(hosts_list)
        ports_string = u','.join(ports_list)
        roots_string = u','.join(roots_list)
        self.__callback(hosts_string, ports_string, roots_string)
        self.__parent.Close()